"""This file imports used classes, modules and packages."""

import ctypes
import importlib
import platform
import sys
from pathlib import Path

if (
    platform.system() == "Windows"
    and sys.version_info.major == 3
    and sys.version_info.minor >= 8
):
    # Starting with Python 3.8, the .dll search mechanism has changed.
    # WinDLL has anew argument "winmode",
    # https://docs.python.org/3.8/library/ctypes.html
    # and it turns out that we MUST import the pybind11 generated module
    # with "winmode=0". After doing this, the following import statement
    # picks this up since it's already imported, and things work as intended.
    #
    # The winmode parameter is used on Windows to specify how the library is
    # loaded (since mode is ignored). It takes any value that is valid for the
    # Win32 API LoadLibraryEx flags parameter. When omitted, the default is to
    # use the flags that result in the most secure DLL load to avoiding issues
    # such as DLL hijacking. Passing winmode=0 passes 0 as dwFlags to
    # LoadLibraryExA:
    # https://docs.microsoft.com/en-us/windows/win32/api/libloaderapi/nf-libloaderapi-loadlibraryexa

    package_dir = Path(importlib.util.find_spec("_zivid").origin).parent
    pyd_files = list(package_dir.glob("_zivid*.pyd"))
    assert len(pyd_files) == 1
    ctypes.WinDLL(str(pyd_files[0]), winmode=0)

try:
    from _zivid._zivid import (  # pylint: disable=import-error,no-name-in-module
        __version__,
        Application,
        Array2DColorRGBA,
        Array2DColorBGRA,
        Array2DColorSRGB,
        Array2DNormalXYZ,
        Array2DPointXYZ,
        Array2DPointXYZColorRGBA,
        Array2DPointXYZColorBGRA,
        Array2DPointXYZW,
        Array2DPointZ,
        Array2DSNR,
        Camera,
        CameraIntrinsics,
        CameraState,
        firmware,
        calibration,
        capture_assistant,
        Frame,
        FrameInfo,
        PointCloud,
        point_cloud_export,
        Settings,
        version,
        Settings2D,
        Frame2D,
        ImageRGBA,
        ImageBGRA,
        ImageSRGB,
        CameraInfo,
        infield_correction,
        Matrix4x4,
        NetworkConfiguration,
        SceneConditions,
        data_model,
        PixelMapping,
        projection,
        ProjectedImage,
        presets,
    )
except ImportError as ex:

    def __missing_sdk_error_message():
        error_message = """Failed to import the Zivid Python C-module, please verify that:
 - Zivid SDK is installed
 - Zivid SDK version is matching the SDK version part of the Zivid Python version """
        if platform.system() != "Windows":
            return error_message
        return (
            error_message
            + """
 - Zivid SDK libraries location is in system PATH"""
        )

    raise ImportError(__missing_sdk_error_message()) from ex



"""Auto generated, do not edit."""

# pylint: disable=too-many-lines,protected-access,too-few-public-methods,too-many-arguments,line-too-long,missing-function-docstring,missing-class-docstring,redefined-builtin,too-many-branches,too-many-boolean-expressions
import datetime
import collections.abc
import zivid.settings2d
import _zivid


class Settings:

    class Acquisition:

        def __init__(
            self,
            aperture=_zivid.Settings.Acquisition.Aperture().value,
            brightness=_zivid.Settings.Acquisition.Brightness().value,
            exposure_time=_zivid.Settings.Acquisition.ExposureTime().value,
            gain=_zivid.Settings.Acquisition.Gain().value,
        ):

            if (
                isinstance(
                    aperture,
                    (
                        float,
                        int,
                    ),
                )
                or aperture is None
            ):
                self._aperture = _zivid.Settings.Acquisition.Aperture(aperture)
            else:
                raise TypeError(
                    "Unsupported type, expected: (float, int,) or None, got {value_type}".format(
                        value_type=type(aperture)
                    )
                )

            if (
                isinstance(
                    brightness,
                    (
                        float,
                        int,
                    ),
                )
                or brightness is None
            ):
                self._brightness = _zivid.Settings.Acquisition.Brightness(brightness)
            else:
                raise TypeError(
                    "Unsupported type, expected: (float, int,) or None, got {value_type}".format(
                        value_type=type(brightness)
                    )
                )

            if (
                isinstance(exposure_time, (datetime.timedelta,))
                or exposure_time is None
            ):
                self._exposure_time = _zivid.Settings.Acquisition.ExposureTime(
                    exposure_time
                )
            else:
                raise TypeError(
                    "Unsupported type, expected: (datetime.timedelta,) or None, got {value_type}".format(
                        value_type=type(exposure_time)
                    )
                )

            if (
                isinstance(
                    gain,
                    (
                        float,
                        int,
                    ),
                )
                or gain is None
            ):
                self._gain = _zivid.Settings.Acquisition.Gain(gain)
            else:
                raise TypeError(
                    "Unsupported type, expected: (float, int,) or None, got {value_type}".format(
                        value_type=type(gain)
                    )
                )

        @property
        def aperture(self):
            return self._aperture.value

        @property
        def brightness(self):
            return self._brightness.value

        @property
        def exposure_time(self):
            return self._exposure_time.value

        @property
        def gain(self):
            return self._gain.value

        @aperture.setter
        def aperture(self, value):
            if (
                isinstance(
                    value,
                    (
                        float,
                        int,
                    ),
                )
                or value is None
            ):
                self._aperture = _zivid.Settings.Acquisition.Aperture(value)
            else:
                raise TypeError(
                    "Unsupported type, expected: float or  int or None, got {value_type}".format(
                        value_type=type(value)
                    )
                )

        @brightness.setter
        def brightness(self, value):
            if (
                isinstance(
                    value,
                    (
                        float,
                        int,
                    ),
                )
                or value is None
            ):
                self._brightness = _zivid.Settings.Acquisition.Brightness(value)
            else:
                raise TypeError(
                    "Unsupported type, expected: float or  int or None, got {value_type}".format(
                        value_type=type(value)
                    )
                )

        @exposure_time.setter
        def exposure_time(self, value):
            if isinstance(value, (datetime.timedelta,)) or value is None:
                self._exposure_time = _zivid.Settings.Acquisition.ExposureTime(value)
            else:
                raise TypeError(
                    "Unsupported type, expected: datetime.timedelta or None, got {value_type}".format(
                        value_type=type(value)
                    )
                )

        @gain.setter
        def gain(self, value):
            if (
                isinstance(
                    value,
                    (
                        float,
                        int,
                    ),
                )
                or value is None
            ):
                self._gain = _zivid.Settings.Acquisition.Gain(value)
            else:
                raise TypeError(
                    "Unsupported type, expected: float or  int or None, got {value_type}".format(
                        value_type=type(value)
                    )
                )

        def __eq__(self, other):
            if (
                self._aperture == other._aperture
                and self._brightness == other._brightness
                and self._exposure_time == other._exposure_time
                and self._gain == other._gain
            ):
                return True
            return False

        def __str__(self):
            return str(_to_internal_settings_acquisition(self))

    class Diagnostics:

        def __init__(
            self,
            enabled=_zivid.Settings.Diagnostics.Enabled().value,
        ):

            if isinstance(enabled, (bool,)) or enabled is None:
                self._enabled = _zivid.Settings.Diagnostics.Enabled(enabled)
            else:
                raise TypeError(
                    "Unsupported type, expected: (bool,) or None, got {value_type}".format(
                        value_type=type(enabled)
                    )
                )

        @property
        def enabled(self):
            return self._enabled.value

        @enabled.setter
        def enabled(self, value):
            if isinstance(value, (bool,)) or value is None:
                self._enabled = _zivid.Settings.Diagnostics.Enabled(value)
            else:
                raise TypeError(
                    "Unsupported type, expected: bool or None, got {value_type}".format(
                        value_type=type(value)
                    )
                )

        def __eq__(self, other):
            if self._enabled == other._enabled:
                return True
            return False

        def __str__(self):
            return str(_to_internal_settings_diagnostics(self))

    class Processing:

        class Color:

            class Balance:

                def __init__(
                    self,
                    blue=_zivid.Settings.Processing.Color.Balance.Blue().value,
                    green=_zivid.Settings.Processing.Color.Balance.Green().value,
                    red=_zivid.Settings.Processing.Color.Balance.Red().value,
                ):

                    if (
                        isinstance(
                            blue,
                            (
                                float,
                                int,
                            ),
                        )
                        or blue is None
                    ):
                        self._blue = _zivid.Settings.Processing.Color.Balance.Blue(blue)
                    else:
                        raise TypeError(
                            "Unsupported type, expected: (float, int,) or None, got {value_type}".format(
                                value_type=type(blue)
                            )
                        )

                    if (
                        isinstance(
                            green,
                            (
                                float,
                                int,
                            ),
                        )
                        or green is None
                    ):
                        self._green = _zivid.Settings.Processing.Color.Balance.Green(
                            green
                        )
                    else:
                        raise TypeError(
                            "Unsupported type, expected: (float, int,) or None, got {value_type}".format(
                                value_type=type(green)
                            )
                        )

                    if (
                        isinstance(
                            red,
                            (
                                float,
                                int,
                            ),
                        )
                        or red is None
                    ):
                        self._red = _zivid.Settings.Processing.Color.Balance.Red(red)
                    else:
                        raise TypeError(
                            "Unsupported type, expected: (float, int,) or None, got {value_type}".format(
                                value_type=type(red)
                            )
                        )

                @property
                def blue(self):
                    return self._blue.value

                @property
                def green(self):
                    return self._green.value

                @property
                def red(self):
                    return self._red.value

                @blue.setter
                def blue(self, value):
                    if (
                        isinstance(
                            value,
                            (
                                float,
                                int,
                            ),
                        )
                        or value is None
                    ):
                        self._blue = _zivid.Settings.Processing.Color.Balance.Blue(
                            value
                        )
                    else:
                        raise TypeError(
                            "Unsupported type, expected: float or  int or None, got {value_type}".format(
                                value_type=type(value)
                            )
                        )

                @green.setter
                def green(self, value):
                    if (
                        isinstance(
                            value,
                            (
                                float,
                                int,
                            ),
                        )
                        or value is None
                    ):
                        self._green = _zivid.Settings.Processing.Color.Balance.Green(
                            value
                        )
                    else:
                        raise TypeError(
                            "Unsupported type, expected: float or  int or None, got {value_type}".format(
                                value_type=type(value)
                            )
                        )

                @red.setter
                def red(self, value):
                    if (
                        isinstance(
                            value,
                            (
                                float,
                                int,
                            ),
                        )
                        or value is None
                    ):
                        self._red = _zivid.Settings.Processing.Color.Balance.Red(value)
                    else:
                        raise TypeError(
                            "Unsupported type, expected: float or  int or None, got {value_type}".format(
                                value_type=type(value)
                            )
                        )

                def __eq__(self, other):
                    if (
                        self._blue == other._blue
                        and self._green == other._green
                        and self._red == other._red
                    ):
                        return True
                    return False

                def __str__(self):
                    return str(_to_internal_settings_processing_color_balance(self))

            class Experimental:

                class Mode:

                    automatic = "automatic"
                    toneMapping = "toneMapping"
                    useFirstAcquisition = "useFirstAcquisition"

                    _valid_values = {
                        "automatic": _zivid.Settings.Processing.Color.Experimental.Mode.automatic,
                        "toneMapping": _zivid.Settings.Processing.Color.Experimental.Mode.toneMapping,
                        "useFirstAcquisition": _zivid.Settings.Processing.Color.Experimental.Mode.useFirstAcquisition,
                    }

                    @classmethod
                    def valid_values(cls):
                        return list(cls._valid_values.keys())

                def __init__(
                    self,
                    mode=_zivid.Settings.Processing.Color.Experimental.Mode().value,
                ):

                    if (
                        isinstance(
                            mode,
                            _zivid.Settings.Processing.Color.Experimental.Mode.enum,
                        )
                        or mode is None
                    ):
                        self._mode = _zivid.Settings.Processing.Color.Experimental.Mode(
                            mode
                        )
                    elif isinstance(mode, str):
                        self._mode = _zivid.Settings.Processing.Color.Experimental.Mode(
                            self.Mode._valid_values[mode]
                        )
                    else:
                        raise TypeError(
                            "Unsupported type, expected: str or None, got {value_type}".format(
                                value_type=type(mode)
                            )
                        )

                @property
                def mode(self):
                    if self._mode.value is None:
                        return None
                    for key, internal_value in self.Mode._valid_values.items():
                        if internal_value == self._mode.value:
                            return key
                    raise ValueError(
                        "Unsupported value {value}".format(value=self._mode)
                    )

                @mode.setter
                def mode(self, value):
                    if isinstance(value, str):
                        self._mode = _zivid.Settings.Processing.Color.Experimental.Mode(
                            self.Mode._valid_values[value]
                        )
                    elif (
                        isinstance(
                            value,
                            _zivid.Settings.Processing.Color.Experimental.Mode.enum,
                        )
                        or value is None
                    ):
                        self._mode = _zivid.Settings.Processing.Color.Experimental.Mode(
                            value
                        )
                    else:
                        raise TypeError(
                            "Unsupported type, expected: str or None, got {value_type}".format(
                                value_type=type(value)
                            )
                        )

                def __eq__(self, other):
                    if self._mode == other._mode:
                        return True
                    return False

                def __str__(self):
                    return str(
                        _to_internal_settings_processing_color_experimental(self)
                    )

            def __init__(
                self,
                gamma=_zivid.Settings.Processing.Color.Gamma().value,
                balance=None,
                experimental=None,
            ):

                if (
                    isinstance(
                        gamma,
                        (
                            float,
                            int,
                        ),
                    )
                    or gamma is None
                ):
                    self._gamma = _zivid.Settings.Processing.Color.Gamma(gamma)
                else:
                    raise TypeError(
                        "Unsupported type, expected: (float, int,) or None, got {value_type}".format(
                            value_type=type(gamma)
                        )
                    )

                if balance is None:
                    balance = self.Balance()
                if not isinstance(balance, self.Balance):
                    raise TypeError(
                        "Unsupported type: {value}".format(value=type(balance))
                    )
                self._balance = balance

                if experimental is None:
                    experimental = self.Experimental()
                if not isinstance(experimental, self.Experimental):
                    raise TypeError(
                        "Unsupported type: {value}".format(value=type(experimental))
                    )
                self._experimental = experimental

            @property
            def gamma(self):
                return self._gamma.value

            @property
            def balance(self):
                return self._balance

            @property
            def experimental(self):
                return self._experimental

            @gamma.setter
            def gamma(self, value):
                if (
                    isinstance(
                        value,
                        (
                            float,
                            int,
                        ),
                    )
                    or value is None
                ):
                    self._gamma = _zivid.Settings.Processing.Color.Gamma(value)
                else:
                    raise TypeError(
                        "Unsupported type, expected: float or  int or None, got {value_type}".format(
                            value_type=type(value)
                        )
                    )

            @balance.setter
            def balance(self, value):
                if not isinstance(value, self.Balance):
                    raise TypeError(
                        "Unsupported type {value}".format(value=type(value))
                    )
                self._balance = value

            @experimental.setter
            def experimental(self, value):
                if not isinstance(value, self.Experimental):
                    raise TypeError(
                        "Unsupported type {value}".format(value=type(value))
                    )
                self._experimental = value

            def __eq__(self, other):
                if (
                    self._gamma == other._gamma
                    and self._balance == other._balance
                    and self._experimental == other._experimental
                ):
                    return True
                return False

            def __str__(self):
                return str(_to_internal_settings_processing_color(self))

        class Filters:

            class Cluster:

                class Removal:

                    def __init__(
                        self,
                        enabled=_zivid.Settings.Processing.Filters.Cluster.Removal.Enabled().value,
                        max_neighbor_distance=_zivid.Settings.Processing.Filters.Cluster.Removal.MaxNeighborDistance().value,
                        min_area=_zivid.Settings.Processing.Filters.Cluster.Removal.MinArea().value,
                    ):

                        if isinstance(enabled, (bool,)) or enabled is None:
                            self._enabled = _zivid.Settings.Processing.Filters.Cluster.Removal.Enabled(
                                enabled
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: (bool,) or None, got {value_type}".format(
                                    value_type=type(enabled)
                                )
                            )

                        if (
                            isinstance(
                                max_neighbor_distance,
                                (
                                    float,
                                    int,
                                ),
                            )
                            or max_neighbor_distance is None
                        ):
                            self._max_neighbor_distance = _zivid.Settings.Processing.Filters.Cluster.Removal.MaxNeighborDistance(
                                max_neighbor_distance
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: (float, int,) or None, got {value_type}".format(
                                    value_type=type(max_neighbor_distance)
                                )
                            )

                        if (
                            isinstance(
                                min_area,
                                (
                                    float,
                                    int,
                                ),
                            )
                            or min_area is None
                        ):
                            self._min_area = _zivid.Settings.Processing.Filters.Cluster.Removal.MinArea(
                                min_area
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: (float, int,) or None, got {value_type}".format(
                                    value_type=type(min_area)
                                )
                            )

                    @property
                    def enabled(self):
                        return self._enabled.value

                    @property
                    def max_neighbor_distance(self):
                        return self._max_neighbor_distance.value

                    @property
                    def min_area(self):
                        return self._min_area.value

                    @enabled.setter
                    def enabled(self, value):
                        if isinstance(value, (bool,)) or value is None:
                            self._enabled = _zivid.Settings.Processing.Filters.Cluster.Removal.Enabled(
                                value
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: bool or None, got {value_type}".format(
                                    value_type=type(value)
                                )
                            )

                    @max_neighbor_distance.setter
                    def max_neighbor_distance(self, value):
                        if (
                            isinstance(
                                value,
                                (
                                    float,
                                    int,
                                ),
                            )
                            or value is None
                        ):
                            self._max_neighbor_distance = _zivid.Settings.Processing.Filters.Cluster.Removal.MaxNeighborDistance(
                                value
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: float or  int or None, got {value_type}".format(
                                    value_type=type(value)
                                )
                            )

                    @min_area.setter
                    def min_area(self, value):
                        if (
                            isinstance(
                                value,
                                (
                                    float,
                                    int,
                                ),
                            )
                            or value is None
                        ):
                            self._min_area = _zivid.Settings.Processing.Filters.Cluster.Removal.MinArea(
                                value
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: float or  int or None, got {value_type}".format(
                                    value_type=type(value)
                                )
                            )

                    def __eq__(self, other):
                        if (
                            self._enabled == other._enabled
                            and self._max_neighbor_distance
                            == other._max_neighbor_distance
                            and self._min_area == other._min_area
                        ):
                            return True
                        return False

                    def __str__(self):
                        return str(
                            _to_internal_settings_processing_filters_cluster_removal(
                                self
                            )
                        )

                def __init__(
                    self,
                    removal=None,
                ):

                    if removal is None:
                        removal = self.Removal()
                    if not isinstance(removal, self.Removal):
                        raise TypeError(
                            "Unsupported type: {value}".format(value=type(removal))
                        )
                    self._removal = removal

                @property
                def removal(self):
                    return self._removal

                @removal.setter
                def removal(self, value):
                    if not isinstance(value, self.Removal):
                        raise TypeError(
                            "Unsupported type {value}".format(value=type(value))
                        )
                    self._removal = value

                def __eq__(self, other):
                    if self._removal == other._removal:
                        return True
                    return False

                def __str__(self):
                    return str(_to_internal_settings_processing_filters_cluster(self))

            class Experimental:

                class ContrastDistortion:

                    class Correction:

                        def __init__(
                            self,
                            enabled=_zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Correction.Enabled().value,
                            strength=_zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Correction.Strength().value,
                        ):

                            if isinstance(enabled, (bool,)) or enabled is None:
                                self._enabled = _zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Correction.Enabled(
                                    enabled
                                )
                            else:
                                raise TypeError(
                                    "Unsupported type, expected: (bool,) or None, got {value_type}".format(
                                        value_type=type(enabled)
                                    )
                                )

                            if (
                                isinstance(
                                    strength,
                                    (
                                        float,
                                        int,
                                    ),
                                )
                                or strength is None
                            ):
                                self._strength = _zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Correction.Strength(
                                    strength
                                )
                            else:
                                raise TypeError(
                                    "Unsupported type, expected: (float, int,) or None, got {value_type}".format(
                                        value_type=type(strength)
                                    )
                                )

                        @property
                        def enabled(self):
                            return self._enabled.value

                        @property
                        def strength(self):
                            return self._strength.value

                        @enabled.setter
                        def enabled(self, value):
                            if isinstance(value, (bool,)) or value is None:
                                self._enabled = _zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Correction.Enabled(
                                    value
                                )
                            else:
                                raise TypeError(
                                    "Unsupported type, expected: bool or None, got {value_type}".format(
                                        value_type=type(value)
                                    )
                                )

                        @strength.setter
                        def strength(self, value):
                            if (
                                isinstance(
                                    value,
                                    (
                                        float,
                                        int,
                                    ),
                                )
                                or value is None
                            ):
                                self._strength = _zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Correction.Strength(
                                    value
                                )
                            else:
                                raise TypeError(
                                    "Unsupported type, expected: float or  int or None, got {value_type}".format(
                                        value_type=type(value)
                                    )
                                )

                        def __eq__(self, other):
                            if (
                                self._enabled == other._enabled
                                and self._strength == other._strength
                            ):
                                return True
                            return False

                        def __str__(self):
                            return str(
                                _to_internal_settings_processing_filters_experimental_contrast_distortion_correction(
                                    self
                                )
                            )

                    class Removal:

                        def __init__(
                            self,
                            enabled=_zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Removal.Enabled().value,
                            threshold=_zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Removal.Threshold().value,
                        ):

                            if isinstance(enabled, (bool,)) or enabled is None:
                                self._enabled = _zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Removal.Enabled(
                                    enabled
                                )
                            else:
                                raise TypeError(
                                    "Unsupported type, expected: (bool,) or None, got {value_type}".format(
                                        value_type=type(enabled)
                                    )
                                )

                            if (
                                isinstance(
                                    threshold,
                                    (
                                        float,
                                        int,
                                    ),
                                )
                                or threshold is None
                            ):
                                self._threshold = _zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Removal.Threshold(
                                    threshold
                                )
                            else:
                                raise TypeError(
                                    "Unsupported type, expected: (float, int,) or None, got {value_type}".format(
                                        value_type=type(threshold)
                                    )
                                )

                        @property
                        def enabled(self):
                            return self._enabled.value

                        @property
                        def threshold(self):
                            return self._threshold.value

                        @enabled.setter
                        def enabled(self, value):
                            if isinstance(value, (bool,)) or value is None:
                                self._enabled = _zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Removal.Enabled(
                                    value
                                )
                            else:
                                raise TypeError(
                                    "Unsupported type, expected: bool or None, got {value_type}".format(
                                        value_type=type(value)
                                    )
                                )

                        @threshold.setter
                        def threshold(self, value):
                            if (
                                isinstance(
                                    value,
                                    (
                                        float,
                                        int,
                                    ),
                                )
                                or value is None
                            ):
                                self._threshold = _zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Removal.Threshold(
                                    value
                                )
                            else:
                                raise TypeError(
                                    "Unsupported type, expected: float or  int or None, got {value_type}".format(
                                        value_type=type(value)
                                    )
                                )

                        def __eq__(self, other):
                            if (
                                self._enabled == other._enabled
                                and self._threshold == other._threshold
                            ):
                                return True
                            return False

                        def __str__(self):
                            return str(
                                _to_internal_settings_processing_filters_experimental_contrast_distortion_removal(
                                    self
                                )
                            )

                    def __init__(
                        self,
                        correction=None,
                        removal=None,
                    ):

                        if correction is None:
                            correction = self.Correction()
                        if not isinstance(correction, self.Correction):
                            raise TypeError(
                                "Unsupported type: {value}".format(
                                    value=type(correction)
                                )
                            )
                        self._correction = correction

                        if removal is None:
                            removal = self.Removal()
                        if not isinstance(removal, self.Removal):
                            raise TypeError(
                                "Unsupported type: {value}".format(value=type(removal))
                            )
                        self._removal = removal

                    @property
                    def correction(self):
                        return self._correction

                    @property
                    def removal(self):
                        return self._removal

                    @correction.setter
                    def correction(self, value):
                        if not isinstance(value, self.Correction):
                            raise TypeError(
                                "Unsupported type {value}".format(value=type(value))
                            )
                        self._correction = value

                    @removal.setter
                    def removal(self, value):
                        if not isinstance(value, self.Removal):
                            raise TypeError(
                                "Unsupported type {value}".format(value=type(value))
                            )
                        self._removal = value

                    def __eq__(self, other):
                        if (
                            self._correction == other._correction
                            and self._removal == other._removal
                        ):
                            return True
                        return False

                    def __str__(self):
                        return str(
                            _to_internal_settings_processing_filters_experimental_contrast_distortion(
                                self
                            )
                        )

                def __init__(
                    self,
                    contrast_distortion=None,
                ):

                    if contrast_distortion is None:
                        contrast_distortion = self.ContrastDistortion()
                    if not isinstance(contrast_distortion, self.ContrastDistortion):
                        raise TypeError(
                            "Unsupported type: {value}".format(
                                value=type(contrast_distortion)
                            )
                        )
                    self._contrast_distortion = contrast_distortion

                @property
                def contrast_distortion(self):
                    return self._contrast_distortion

                @contrast_distortion.setter
                def contrast_distortion(self, value):
                    if not isinstance(value, self.ContrastDistortion):
                        raise TypeError(
                            "Unsupported type {value}".format(value=type(value))
                        )
                    self._contrast_distortion = value

                def __eq__(self, other):
                    if self._contrast_distortion == other._contrast_distortion:
                        return True
                    return False

                def __str__(self):
                    return str(
                        _to_internal_settings_processing_filters_experimental(self)
                    )

            class Hole:

                class Repair:

                    def __init__(
                        self,
                        enabled=_zivid.Settings.Processing.Filters.Hole.Repair.Enabled().value,
                        hole_size=_zivid.Settings.Processing.Filters.Hole.Repair.HoleSize().value,
                        strictness=_zivid.Settings.Processing.Filters.Hole.Repair.Strictness().value,
                    ):

                        if isinstance(enabled, (bool,)) or enabled is None:
                            self._enabled = (
                                _zivid.Settings.Processing.Filters.Hole.Repair.Enabled(
                                    enabled
                                )
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: (bool,) or None, got {value_type}".format(
                                    value_type=type(enabled)
                                )
                            )

                        if (
                            isinstance(
                                hole_size,
                                (
                                    float,
                                    int,
                                ),
                            )
                            or hole_size is None
                        ):
                            self._hole_size = (
                                _zivid.Settings.Processing.Filters.Hole.Repair.HoleSize(
                                    hole_size
                                )
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: (float, int,) or None, got {value_type}".format(
                                    value_type=type(hole_size)
                                )
                            )

                        if isinstance(strictness, (int,)) or strictness is None:
                            self._strictness = _zivid.Settings.Processing.Filters.Hole.Repair.Strictness(
                                strictness
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: (int,) or None, got {value_type}".format(
                                    value_type=type(strictness)
                                )
                            )

                    @property
                    def enabled(self):
                        return self._enabled.value

                    @property
                    def hole_size(self):
                        return self._hole_size.value

                    @property
                    def strictness(self):
                        return self._strictness.value

                    @enabled.setter
                    def enabled(self, value):
                        if isinstance(value, (bool,)) or value is None:
                            self._enabled = (
                                _zivid.Settings.Processing.Filters.Hole.Repair.Enabled(
                                    value
                                )
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: bool or None, got {value_type}".format(
                                    value_type=type(value)
                                )
                            )

                    @hole_size.setter
                    def hole_size(self, value):
                        if (
                            isinstance(
                                value,
                                (
                                    float,
                                    int,
                                ),
                            )
                            or value is None
                        ):
                            self._hole_size = (
                                _zivid.Settings.Processing.Filters.Hole.Repair.HoleSize(
                                    value
                                )
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: float or  int or None, got {value_type}".format(
                                    value_type=type(value)
                                )
                            )

                    @strictness.setter
                    def strictness(self, value):
                        if isinstance(value, (int,)) or value is None:
                            self._strictness = _zivid.Settings.Processing.Filters.Hole.Repair.Strictness(
                                value
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: int or None, got {value_type}".format(
                                    value_type=type(value)
                                )
                            )

                    def __eq__(self, other):
                        if (
                            self._enabled == other._enabled
                            and self._hole_size == other._hole_size
                            and self._strictness == other._strictness
                        ):
                            return True
                        return False

                    def __str__(self):
                        return str(
                            _to_internal_settings_processing_filters_hole_repair(self)
                        )

                def __init__(
                    self,
                    repair=None,
                ):

                    if repair is None:
                        repair = self.Repair()
                    if not isinstance(repair, self.Repair):
                        raise TypeError(
                            "Unsupported type: {value}".format(value=type(repair))
                        )
                    self._repair = repair

                @property
                def repair(self):
                    return self._repair

                @repair.setter
                def repair(self, value):
                    if not isinstance(value, self.Repair):
                        raise TypeError(
                            "Unsupported type {value}".format(value=type(value))
                        )
                    self._repair = value

                def __eq__(self, other):
                    if self._repair == other._repair:
                        return True
                    return False

                def __str__(self):
                    return str(_to_internal_settings_processing_filters_hole(self))

            class Noise:

                class Removal:

                    def __init__(
                        self,
                        enabled=_zivid.Settings.Processing.Filters.Noise.Removal.Enabled().value,
                        threshold=_zivid.Settings.Processing.Filters.Noise.Removal.Threshold().value,
                    ):

                        if isinstance(enabled, (bool,)) or enabled is None:
                            self._enabled = _zivid.Settings.Processing.Filters.Noise.Removal.Enabled(
                                enabled
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: (bool,) or None, got {value_type}".format(
                                    value_type=type(enabled)
                                )
                            )

                        if (
                            isinstance(
                                threshold,
                                (
                                    float,
                                    int,
                                ),
                            )
                            or threshold is None
                        ):
                            self._threshold = _zivid.Settings.Processing.Filters.Noise.Removal.Threshold(
                                threshold
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: (float, int,) or None, got {value_type}".format(
                                    value_type=type(threshold)
                                )
                            )

                    @property
                    def enabled(self):
                        return self._enabled.value

                    @property
                    def threshold(self):
                        return self._threshold.value

                    @enabled.setter
                    def enabled(self, value):
                        if isinstance(value, (bool,)) or value is None:
                            self._enabled = _zivid.Settings.Processing.Filters.Noise.Removal.Enabled(
                                value
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: bool or None, got {value_type}".format(
                                    value_type=type(value)
                                )
                            )

                    @threshold.setter
                    def threshold(self, value):
                        if (
                            isinstance(
                                value,
                                (
                                    float,
                                    int,
                                ),
                            )
                            or value is None
                        ):
                            self._threshold = _zivid.Settings.Processing.Filters.Noise.Removal.Threshold(
                                value
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: float or  int or None, got {value_type}".format(
                                    value_type=type(value)
                                )
                            )

                    def __eq__(self, other):
                        if (
                            self._enabled == other._enabled
                            and self._threshold == other._threshold
                        ):
                            return True
                        return False

                    def __str__(self):
                        return str(
                            _to_internal_settings_processing_filters_noise_removal(self)
                        )

                class Repair:

                    def __init__(
                        self,
                        enabled=_zivid.Settings.Processing.Filters.Noise.Repair.Enabled().value,
                    ):

                        if isinstance(enabled, (bool,)) or enabled is None:
                            self._enabled = (
                                _zivid.Settings.Processing.Filters.Noise.Repair.Enabled(
                                    enabled
                                )
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: (bool,) or None, got {value_type}".format(
                                    value_type=type(enabled)
                                )
                            )

                    @property
                    def enabled(self):
                        return self._enabled.value

                    @enabled.setter
                    def enabled(self, value):
                        if isinstance(value, (bool,)) or value is None:
                            self._enabled = (
                                _zivid.Settings.Processing.Filters.Noise.Repair.Enabled(
                                    value
                                )
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: bool or None, got {value_type}".format(
                                    value_type=type(value)
                                )
                            )

                    def __eq__(self, other):
                        if self._enabled == other._enabled:
                            return True
                        return False

                    def __str__(self):
                        return str(
                            _to_internal_settings_processing_filters_noise_repair(self)
                        )

                class Suppression:

                    def __init__(
                        self,
                        enabled=_zivid.Settings.Processing.Filters.Noise.Suppression.Enabled().value,
                    ):

                        if isinstance(enabled, (bool,)) or enabled is None:
                            self._enabled = _zivid.Settings.Processing.Filters.Noise.Suppression.Enabled(
                                enabled
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: (bool,) or None, got {value_type}".format(
                                    value_type=type(enabled)
                                )
                            )

                    @property
                    def enabled(self):
                        return self._enabled.value

                    @enabled.setter
                    def enabled(self, value):
                        if isinstance(value, (bool,)) or value is None:
                            self._enabled = _zivid.Settings.Processing.Filters.Noise.Suppression.Enabled(
                                value
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: bool or None, got {value_type}".format(
                                    value_type=type(value)
                                )
                            )

                    def __eq__(self, other):
                        if self._enabled == other._enabled:
                            return True
                        return False

                    def __str__(self):
                        return str(
                            _to_internal_settings_processing_filters_noise_suppression(
                                self
                            )
                        )

                def __init__(
                    self,
                    removal=None,
                    repair=None,
                    suppression=None,
                ):

                    if removal is None:
                        removal = self.Removal()
                    if not isinstance(removal, self.Removal):
                        raise TypeError(
                            "Unsupported type: {value}".format(value=type(removal))
                        )
                    self._removal = removal

                    if repair is None:
                        repair = self.Repair()
                    if not isinstance(repair, self.Repair):
                        raise TypeError(
                            "Unsupported type: {value}".format(value=type(repair))
                        )
                    self._repair = repair

                    if suppression is None:
                        suppression = self.Suppression()
                    if not isinstance(suppression, self.Suppression):
                        raise TypeError(
                            "Unsupported type: {value}".format(value=type(suppression))
                        )
                    self._suppression = suppression

                @property
                def removal(self):
                    return self._removal

                @property
                def repair(self):
                    return self._repair

                @property
                def suppression(self):
                    return self._suppression

                @removal.setter
                def removal(self, value):
                    if not isinstance(value, self.Removal):
                        raise TypeError(
                            "Unsupported type {value}".format(value=type(value))
                        )
                    self._removal = value

                @repair.setter
                def repair(self, value):
                    if not isinstance(value, self.Repair):
                        raise TypeError(
                            "Unsupported type {value}".format(value=type(value))
                        )
                    self._repair = value

                @suppression.setter
                def suppression(self, value):
                    if not isinstance(value, self.Suppression):
                        raise TypeError(
                            "Unsupported type {value}".format(value=type(value))
                        )
                    self._suppression = value

                def __eq__(self, other):
                    if (
                        self._removal == other._removal
                        and self._repair == other._repair
                        and self._suppression == other._suppression
                    ):
                        return True
                    return False

                def __str__(self):
                    return str(_to_internal_settings_processing_filters_noise(self))

            class Outlier:

                class Removal:

                    def __init__(
                        self,
                        enabled=_zivid.Settings.Processing.Filters.Outlier.Removal.Enabled().value,
                        threshold=_zivid.Settings.Processing.Filters.Outlier.Removal.Threshold().value,
                    ):

                        if isinstance(enabled, (bool,)) or enabled is None:
                            self._enabled = _zivid.Settings.Processing.Filters.Outlier.Removal.Enabled(
                                enabled
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: (bool,) or None, got {value_type}".format(
                                    value_type=type(enabled)
                                )
                            )

                        if (
                            isinstance(
                                threshold,
                                (
                                    float,
                                    int,
                                ),
                            )
                            or threshold is None
                        ):
                            self._threshold = _zivid.Settings.Processing.Filters.Outlier.Removal.Threshold(
                                threshold
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: (float, int,) or None, got {value_type}".format(
                                    value_type=type(threshold)
                                )
                            )

                    @property
                    def enabled(self):
                        return self._enabled.value

                    @property
                    def threshold(self):
                        return self._threshold.value

                    @enabled.setter
                    def enabled(self, value):
                        if isinstance(value, (bool,)) or value is None:
                            self._enabled = _zivid.Settings.Processing.Filters.Outlier.Removal.Enabled(
                                value
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: bool or None, got {value_type}".format(
                                    value_type=type(value)
                                )
                            )

                    @threshold.setter
                    def threshold(self, value):
                        if (
                            isinstance(
                                value,
                                (
                                    float,
                                    int,
                                ),
                            )
                            or value is None
                        ):
                            self._threshold = _zivid.Settings.Processing.Filters.Outlier.Removal.Threshold(
                                value
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: float or  int or None, got {value_type}".format(
                                    value_type=type(value)
                                )
                            )

                    def __eq__(self, other):
                        if (
                            self._enabled == other._enabled
                            and self._threshold == other._threshold
                        ):
                            return True
                        return False

                    def __str__(self):
                        return str(
                            _to_internal_settings_processing_filters_outlier_removal(
                                self
                            )
                        )

                def __init__(
                    self,
                    removal=None,
                ):

                    if removal is None:
                        removal = self.Removal()
                    if not isinstance(removal, self.Removal):
                        raise TypeError(
                            "Unsupported type: {value}".format(value=type(removal))
                        )
                    self._removal = removal

                @property
                def removal(self):
                    return self._removal

                @removal.setter
                def removal(self, value):
                    if not isinstance(value, self.Removal):
                        raise TypeError(
                            "Unsupported type {value}".format(value=type(value))
                        )
                    self._removal = value

                def __eq__(self, other):
                    if self._removal == other._removal:
                        return True
                    return False

                def __str__(self):
                    return str(_to_internal_settings_processing_filters_outlier(self))

            class Reflection:

                class Removal:

                    class Mode:

                        global_ = "global"
                        local = "local"

                        _valid_values = {
                            "global": _zivid.Settings.Processing.Filters.Reflection.Removal.Mode.global_,
                            "local": _zivid.Settings.Processing.Filters.Reflection.Removal.Mode.local,
                        }

                        @classmethod
                        def valid_values(cls):
                            return list(cls._valid_values.keys())

                    def __init__(
                        self,
                        enabled=_zivid.Settings.Processing.Filters.Reflection.Removal.Enabled().value,
                        mode=_zivid.Settings.Processing.Filters.Reflection.Removal.Mode().value,
                    ):

                        if isinstance(enabled, (bool,)) or enabled is None:
                            self._enabled = _zivid.Settings.Processing.Filters.Reflection.Removal.Enabled(
                                enabled
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: (bool,) or None, got {value_type}".format(
                                    value_type=type(enabled)
                                )
                            )

                        if (
                            isinstance(
                                mode,
                                _zivid.Settings.Processing.Filters.Reflection.Removal.Mode.enum,
                            )
                            or mode is None
                        ):
                            self._mode = _zivid.Settings.Processing.Filters.Reflection.Removal.Mode(
                                mode
                            )
                        elif isinstance(mode, str):
                            self._mode = _zivid.Settings.Processing.Filters.Reflection.Removal.Mode(
                                self.Mode._valid_values[mode]
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: str or None, got {value_type}".format(
                                    value_type=type(mode)
                                )
                            )

                    @property
                    def enabled(self):
                        return self._enabled.value

                    @property
                    def mode(self):
                        if self._mode.value is None:
                            return None
                        for key, internal_value in self.Mode._valid_values.items():
                            if internal_value == self._mode.value:
                                return key
                        raise ValueError(
                            "Unsupported value {value}".format(value=self._mode)
                        )

                    @enabled.setter
                    def enabled(self, value):
                        if isinstance(value, (bool,)) or value is None:
                            self._enabled = _zivid.Settings.Processing.Filters.Reflection.Removal.Enabled(
                                value
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: bool or None, got {value_type}".format(
                                    value_type=type(value)
                                )
                            )

                    @mode.setter
                    def mode(self, value):
                        if isinstance(value, str):
                            self._mode = _zivid.Settings.Processing.Filters.Reflection.Removal.Mode(
                                self.Mode._valid_values[value]
                            )
                        elif (
                            isinstance(
                                value,
                                _zivid.Settings.Processing.Filters.Reflection.Removal.Mode.enum,
                            )
                            or value is None
                        ):
                            self._mode = _zivid.Settings.Processing.Filters.Reflection.Removal.Mode(
                                value
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: str or None, got {value_type}".format(
                                    value_type=type(value)
                                )
                            )

                    def __eq__(self, other):
                        if (
                            self._enabled == other._enabled
                            and self._mode == other._mode
                        ):
                            return True
                        return False

                    def __str__(self):
                        return str(
                            _to_internal_settings_processing_filters_reflection_removal(
                                self
                            )
                        )

                def __init__(
                    self,
                    removal=None,
                ):

                    if removal is None:
                        removal = self.Removal()
                    if not isinstance(removal, self.Removal):
                        raise TypeError(
                            "Unsupported type: {value}".format(value=type(removal))
                        )
                    self._removal = removal

                @property
                def removal(self):
                    return self._removal

                @removal.setter
                def removal(self, value):
                    if not isinstance(value, self.Removal):
                        raise TypeError(
                            "Unsupported type {value}".format(value=type(value))
                        )
                    self._removal = value

                def __eq__(self, other):
                    if self._removal == other._removal:
                        return True
                    return False

                def __str__(self):
                    return str(
                        _to_internal_settings_processing_filters_reflection(self)
                    )

            class Smoothing:

                class Gaussian:

                    def __init__(
                        self,
                        enabled=_zivid.Settings.Processing.Filters.Smoothing.Gaussian.Enabled().value,
                        sigma=_zivid.Settings.Processing.Filters.Smoothing.Gaussian.Sigma().value,
                    ):

                        if isinstance(enabled, (bool,)) or enabled is None:
                            self._enabled = _zivid.Settings.Processing.Filters.Smoothing.Gaussian.Enabled(
                                enabled
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: (bool,) or None, got {value_type}".format(
                                    value_type=type(enabled)
                                )
                            )

                        if (
                            isinstance(
                                sigma,
                                (
                                    float,
                                    int,
                                ),
                            )
                            or sigma is None
                        ):
                            self._sigma = _zivid.Settings.Processing.Filters.Smoothing.Gaussian.Sigma(
                                sigma
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: (float, int,) or None, got {value_type}".format(
                                    value_type=type(sigma)
                                )
                            )

                    @property
                    def enabled(self):
                        return self._enabled.value

                    @property
                    def sigma(self):
                        return self._sigma.value

                    @enabled.setter
                    def enabled(self, value):
                        if isinstance(value, (bool,)) or value is None:
                            self._enabled = _zivid.Settings.Processing.Filters.Smoothing.Gaussian.Enabled(
                                value
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: bool or None, got {value_type}".format(
                                    value_type=type(value)
                                )
                            )

                    @sigma.setter
                    def sigma(self, value):
                        if (
                            isinstance(
                                value,
                                (
                                    float,
                                    int,
                                ),
                            )
                            or value is None
                        ):
                            self._sigma = _zivid.Settings.Processing.Filters.Smoothing.Gaussian.Sigma(
                                value
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: float or  int or None, got {value_type}".format(
                                    value_type=type(value)
                                )
                            )

                    def __eq__(self, other):
                        if (
                            self._enabled == other._enabled
                            and self._sigma == other._sigma
                        ):
                            return True
                        return False

                    def __str__(self):
                        return str(
                            _to_internal_settings_processing_filters_smoothing_gaussian(
                                self
                            )
                        )

                def __init__(
                    self,
                    gaussian=None,
                ):

                    if gaussian is None:
                        gaussian = self.Gaussian()
                    if not isinstance(gaussian, self.Gaussian):
                        raise TypeError(
                            "Unsupported type: {value}".format(value=type(gaussian))
                        )
                    self._gaussian = gaussian

                @property
                def gaussian(self):
                    return self._gaussian

                @gaussian.setter
                def gaussian(self, value):
                    if not isinstance(value, self.Gaussian):
                        raise TypeError(
                            "Unsupported type {value}".format(value=type(value))
                        )
                    self._gaussian = value

                def __eq__(self, other):
                    if self._gaussian == other._gaussian:
                        return True
                    return False

                def __str__(self):
                    return str(_to_internal_settings_processing_filters_smoothing(self))

            def __init__(
                self,
                cluster=None,
                experimental=None,
                hole=None,
                noise=None,
                outlier=None,
                reflection=None,
                smoothing=None,
            ):

                if cluster is None:
                    cluster = self.Cluster()
                if not isinstance(cluster, self.Cluster):
                    raise TypeError(
                        "Unsupported type: {value}".format(value=type(cluster))
                    )
                self._cluster = cluster

                if experimental is None:
                    experimental = self.Experimental()
                if not isinstance(experimental, self.Experimental):
                    raise TypeError(
                        "Unsupported type: {value}".format(value=type(experimental))
                    )
                self._experimental = experimental

                if hole is None:
                    hole = self.Hole()
                if not isinstance(hole, self.Hole):
                    raise TypeError(
                        "Unsupported type: {value}".format(value=type(hole))
                    )
                self._hole = hole

                if noise is None:
                    noise = self.Noise()
                if not isinstance(noise, self.Noise):
                    raise TypeError(
                        "Unsupported type: {value}".format(value=type(noise))
                    )
                self._noise = noise

                if outlier is None:
                    outlier = self.Outlier()
                if not isinstance(outlier, self.Outlier):
                    raise TypeError(
                        "Unsupported type: {value}".format(value=type(outlier))
                    )
                self._outlier = outlier

                if reflection is None:
                    reflection = self.Reflection()
                if not isinstance(reflection, self.Reflection):
                    raise TypeError(
                        "Unsupported type: {value}".format(value=type(reflection))
                    )
                self._reflection = reflection

                if smoothing is None:
                    smoothing = self.Smoothing()
                if not isinstance(smoothing, self.Smoothing):
                    raise TypeError(
                        "Unsupported type: {value}".format(value=type(smoothing))
                    )
                self._smoothing = smoothing

            @property
            def cluster(self):
                return self._cluster

            @property
            def experimental(self):
                return self._experimental

            @property
            def hole(self):
                return self._hole

            @property
            def noise(self):
                return self._noise

            @property
            def outlier(self):
                return self._outlier

            @property
            def reflection(self):
                return self._reflection

            @property
            def smoothing(self):
                return self._smoothing

            @cluster.setter
            def cluster(self, value):
                if not isinstance(value, self.Cluster):
                    raise TypeError(
                        "Unsupported type {value}".format(value=type(value))
                    )
                self._cluster = value

            @experimental.setter
            def experimental(self, value):
                if not isinstance(value, self.Experimental):
                    raise TypeError(
                        "Unsupported type {value}".format(value=type(value))
                    )
                self._experimental = value

            @hole.setter
            def hole(self, value):
                if not isinstance(value, self.Hole):
                    raise TypeError(
                        "Unsupported type {value}".format(value=type(value))
                    )
                self._hole = value

            @noise.setter
            def noise(self, value):
                if not isinstance(value, self.Noise):
                    raise TypeError(
                        "Unsupported type {value}".format(value=type(value))
                    )
                self._noise = value

            @outlier.setter
            def outlier(self, value):
                if not isinstance(value, self.Outlier):
                    raise TypeError(
                        "Unsupported type {value}".format(value=type(value))
                    )
                self._outlier = value

            @reflection.setter
            def reflection(self, value):
                if not isinstance(value, self.Reflection):
                    raise TypeError(
                        "Unsupported type {value}".format(value=type(value))
                    )
                self._reflection = value

            @smoothing.setter
            def smoothing(self, value):
                if not isinstance(value, self.Smoothing):
                    raise TypeError(
                        "Unsupported type {value}".format(value=type(value))
                    )
                self._smoothing = value

            def __eq__(self, other):
                if (
                    self._cluster == other._cluster
                    and self._experimental == other._experimental
                    and self._hole == other._hole
                    and self._noise == other._noise
                    and self._outlier == other._outlier
                    and self._reflection == other._reflection
                    and self._smoothing == other._smoothing
                ):
                    return True
                return False

            def __str__(self):
                return str(_to_internal_settings_processing_filters(self))

        class Resampling:

            class Mode:

                disabled = "disabled"
                downsample2x2 = "downsample2x2"
                downsample4x4 = "downsample4x4"
                upsample2x2 = "upsample2x2"
                upsample4x4 = "upsample4x4"

                _valid_values = {
                    "disabled": _zivid.Settings.Processing.Resampling.Mode.disabled,
                    "downsample2x2": _zivid.Settings.Processing.Resampling.Mode.downsample2x2,
                    "downsample4x4": _zivid.Settings.Processing.Resampling.Mode.downsample4x4,
                    "upsample2x2": _zivid.Settings.Processing.Resampling.Mode.upsample2x2,
                    "upsample4x4": _zivid.Settings.Processing.Resampling.Mode.upsample4x4,
                }

                @classmethod
                def valid_values(cls):
                    return list(cls._valid_values.keys())

            def __init__(
                self,
                mode=_zivid.Settings.Processing.Resampling.Mode().value,
            ):

                if (
                    isinstance(mode, _zivid.Settings.Processing.Resampling.Mode.enum)
                    or mode is None
                ):
                    self._mode = _zivid.Settings.Processing.Resampling.Mode(mode)
                elif isinstance(mode, str):
                    self._mode = _zivid.Settings.Processing.Resampling.Mode(
                        self.Mode._valid_values[mode]
                    )
                else:
                    raise TypeError(
                        "Unsupported type, expected: str or None, got {value_type}".format(
                            value_type=type(mode)
                        )
                    )

            @property
            def mode(self):
                if self._mode.value is None:
                    return None
                for key, internal_value in self.Mode._valid_values.items():
                    if internal_value == self._mode.value:
                        return key
                raise ValueError("Unsupported value {value}".format(value=self._mode))

            @mode.setter
            def mode(self, value):
                if isinstance(value, str):
                    self._mode = _zivid.Settings.Processing.Resampling.Mode(
                        self.Mode._valid_values[value]
                    )
                elif (
                    isinstance(value, _zivid.Settings.Processing.Resampling.Mode.enum)
                    or value is None
                ):
                    self._mode = _zivid.Settings.Processing.Resampling.Mode(value)
                else:
                    raise TypeError(
                        "Unsupported type, expected: str or None, got {value_type}".format(
                            value_type=type(value)
                        )
                    )

            def __eq__(self, other):
                if self._mode == other._mode:
                    return True
                return False

            def __str__(self):
                return str(_to_internal_settings_processing_resampling(self))

        def __init__(
            self,
            color=None,
            filters=None,
            resampling=None,
        ):

            if color is None:
                color = self.Color()
            if not isinstance(color, self.Color):
                raise TypeError("Unsupported type: {value}".format(value=type(color)))
            self._color = color

            if filters is None:
                filters = self.Filters()
            if not isinstance(filters, self.Filters):
                raise TypeError("Unsupported type: {value}".format(value=type(filters)))
            self._filters = filters

            if resampling is None:
                resampling = self.Resampling()
            if not isinstance(resampling, self.Resampling):
                raise TypeError(
                    "Unsupported type: {value}".format(value=type(resampling))
                )
            self._resampling = resampling

        @property
        def color(self):
            return self._color

        @property
        def filters(self):
            return self._filters

        @property
        def resampling(self):
            return self._resampling

        @color.setter
        def color(self, value):
            if not isinstance(value, self.Color):
                raise TypeError("Unsupported type {value}".format(value=type(value)))
            self._color = value

        @filters.setter
        def filters(self, value):
            if not isinstance(value, self.Filters):
                raise TypeError("Unsupported type {value}".format(value=type(value)))
            self._filters = value

        @resampling.setter
        def resampling(self, value):
            if not isinstance(value, self.Resampling):
                raise TypeError("Unsupported type {value}".format(value=type(value)))
            self._resampling = value

        def __eq__(self, other):
            if (
                self._color == other._color
                and self._filters == other._filters
                and self._resampling == other._resampling
            ):
                return True
            return False

        def __str__(self):
            return str(_to_internal_settings_processing(self))

    class RegionOfInterest:

        class Box:

            def __init__(
                self,
                enabled=_zivid.Settings.RegionOfInterest.Box.Enabled().value,
                extents=_zivid.Settings.RegionOfInterest.Box.Extents().value,
                point_a=_zivid.Settings.RegionOfInterest.Box.PointA().value,
                point_b=_zivid.Settings.RegionOfInterest.Box.PointB().value,
                point_o=_zivid.Settings.RegionOfInterest.Box.PointO().value,
            ):

                if isinstance(enabled, (bool,)) or enabled is None:
                    self._enabled = _zivid.Settings.RegionOfInterest.Box.Enabled(
                        enabled
                    )
                else:
                    raise TypeError(
                        "Unsupported type, expected: (bool,) or None, got {value_type}".format(
                            value_type=type(enabled)
                        )
                    )

                if (
                    isinstance(
                        extents, (collections.abc.Iterable, _zivid.data_model.Range)
                    )
                    or extents is None
                ):
                    self._extents = _zivid.Settings.RegionOfInterest.Box.Extents(
                        extents
                    )
                else:
                    raise TypeError(
                        "Unsupported type, expected: (collections.abc.Iterable, _zivid.data_model.Range) or None, got {value_type}".format(
                            value_type=type(extents)
                        )
                    )

                if (
                    isinstance(
                        point_a, (collections.abc.Iterable, _zivid.data_model.PointXYZ)
                    )
                    or point_a is None
                ):
                    self._point_a = _zivid.Settings.RegionOfInterest.Box.PointA(point_a)
                else:
                    raise TypeError(
                        "Unsupported type, expected: (collections.abc.Iterable, _zivid.data_model.PointXYZ) or None, got {value_type}".format(
                            value_type=type(point_a)
                        )
                    )

                if (
                    isinstance(
                        point_b, (collections.abc.Iterable, _zivid.data_model.PointXYZ)
                    )
                    or point_b is None
                ):
                    self._point_b = _zivid.Settings.RegionOfInterest.Box.PointB(point_b)
                else:
                    raise TypeError(
                        "Unsupported type, expected: (collections.abc.Iterable, _zivid.data_model.PointXYZ) or None, got {value_type}".format(
                            value_type=type(point_b)
                        )
                    )

                if (
                    isinstance(
                        point_o, (collections.abc.Iterable, _zivid.data_model.PointXYZ)
                    )
                    or point_o is None
                ):
                    self._point_o = _zivid.Settings.RegionOfInterest.Box.PointO(point_o)
                else:
                    raise TypeError(
                        "Unsupported type, expected: (collections.abc.Iterable, _zivid.data_model.PointXYZ) or None, got {value_type}".format(
                            value_type=type(point_o)
                        )
                    )

            @property
            def enabled(self):
                return self._enabled.value

            @property
            def extents(self):
                if self._extents.value is None:
                    return None
                return self._extents.value.to_array()

            @property
            def point_a(self):
                if self._point_a.value is None:
                    return None
                return self._point_a.value.to_array()

            @property
            def point_b(self):
                if self._point_b.value is None:
                    return None
                return self._point_b.value.to_array()

            @property
            def point_o(self):
                if self._point_o.value is None:
                    return None
                return self._point_o.value.to_array()

            @enabled.setter
            def enabled(self, value):
                if isinstance(value, (bool,)) or value is None:
                    self._enabled = _zivid.Settings.RegionOfInterest.Box.Enabled(value)
                else:
                    raise TypeError(
                        "Unsupported type, expected: bool or None, got {value_type}".format(
                            value_type=type(value)
                        )
                    )

            @extents.setter
            def extents(self, value):
                if (
                    isinstance(
                        value, (collections.abc.Iterable, _zivid.data_model.Range)
                    )
                    or value is None
                ):
                    self._extents = _zivid.Settings.RegionOfInterest.Box.Extents(value)
                else:
                    raise TypeError(
                        "Unsupported type, expected: collections.abc.Iterable or  _zivid.data_model.Rang or None, got {value_type}".format(
                            value_type=type(value)
                        )
                    )

            @point_a.setter
            def point_a(self, value):
                if (
                    isinstance(
                        value, (collections.abc.Iterable, _zivid.data_model.PointXYZ)
                    )
                    or value is None
                ):
                    self._point_a = _zivid.Settings.RegionOfInterest.Box.PointA(value)
                else:
                    raise TypeError(
                        "Unsupported type, expected: collections.abc.Iterable or  _zivid.data_model.PointXY or None, got {value_type}".format(
                            value_type=type(value)
                        )
                    )

            @point_b.setter
            def point_b(self, value):
                if (
                    isinstance(
                        value, (collections.abc.Iterable, _zivid.data_model.PointXYZ)
                    )
                    or value is None
                ):
                    self._point_b = _zivid.Settings.RegionOfInterest.Box.PointB(value)
                else:
                    raise TypeError(
                        "Unsupported type, expected: collections.abc.Iterable or  _zivid.data_model.PointXY or None, got {value_type}".format(
                            value_type=type(value)
                        )
                    )

            @point_o.setter
            def point_o(self, value):
                if (
                    isinstance(
                        value, (collections.abc.Iterable, _zivid.data_model.PointXYZ)
                    )
                    or value is None
                ):
                    self._point_o = _zivid.Settings.RegionOfInterest.Box.PointO(value)
                else:
                    raise TypeError(
                        "Unsupported type, expected: collections.abc.Iterable or  _zivid.data_model.PointXY or None, got {value_type}".format(
                            value_type=type(value)
                        )
                    )

            def __eq__(self, other):
                if (
                    self._enabled == other._enabled
                    and self._extents == other._extents
                    and self._point_a == other._point_a
                    and self._point_b == other._point_b
                    and self._point_o == other._point_o
                ):
                    return True
                return False

            def __str__(self):
                return str(_to_internal_settings_region_of_interest_box(self))

        class Depth:

            def __init__(
                self,
                enabled=_zivid.Settings.RegionOfInterest.Depth.Enabled().value,
                range=_zivid.Settings.RegionOfInterest.Depth.Range().value,
            ):

                if isinstance(enabled, (bool,)) or enabled is None:
                    self._enabled = _zivid.Settings.RegionOfInterest.Depth.Enabled(
                        enabled
                    )
                else:
                    raise TypeError(
                        "Unsupported type, expected: (bool,) or None, got {value_type}".format(
                            value_type=type(enabled)
                        )
                    )

                if (
                    isinstance(
                        range, (collections.abc.Iterable, _zivid.data_model.Range)
                    )
                    or range is None
                ):
                    self._range = _zivid.Settings.RegionOfInterest.Depth.Range(range)
                else:
                    raise TypeError(
                        "Unsupported type, expected: (collections.abc.Iterable, _zivid.data_model.Range) or None, got {value_type}".format(
                            value_type=type(range)
                        )
                    )

            @property
            def enabled(self):
                return self._enabled.value

            @property
            def range(self):
                if self._range.value is None:
                    return None
                return self._range.value.to_array()

            @enabled.setter
            def enabled(self, value):
                if isinstance(value, (bool,)) or value is None:
                    self._enabled = _zivid.Settings.RegionOfInterest.Depth.Enabled(
                        value
                    )
                else:
                    raise TypeError(
                        "Unsupported type, expected: bool or None, got {value_type}".format(
                            value_type=type(value)
                        )
                    )

            @range.setter
            def range(self, value):
                if (
                    isinstance(
                        value, (collections.abc.Iterable, _zivid.data_model.Range)
                    )
                    or value is None
                ):
                    self._range = _zivid.Settings.RegionOfInterest.Depth.Range(value)
                else:
                    raise TypeError(
                        "Unsupported type, expected: collections.abc.Iterable or  _zivid.data_model.Rang or None, got {value_type}".format(
                            value_type=type(value)
                        )
                    )

            def __eq__(self, other):
                if self._enabled == other._enabled and self._range == other._range:
                    return True
                return False

            def __str__(self):
                return str(_to_internal_settings_region_of_interest_depth(self))

        def __init__(
            self,
            box=None,
            depth=None,
        ):

            if box is None:
                box = self.Box()
            if not isinstance(box, self.Box):
                raise TypeError("Unsupported type: {value}".format(value=type(box)))
            self._box = box

            if depth is None:
                depth = self.Depth()
            if not isinstance(depth, self.Depth):
                raise TypeError("Unsupported type: {value}".format(value=type(depth)))
            self._depth = depth

        @property
        def box(self):
            return self._box

        @property
        def depth(self):
            return self._depth

        @box.setter
        def box(self, value):
            if not isinstance(value, self.Box):
                raise TypeError("Unsupported type {value}".format(value=type(value)))
            self._box = value

        @depth.setter
        def depth(self, value):
            if not isinstance(value, self.Depth):
                raise TypeError("Unsupported type {value}".format(value=type(value)))
            self._depth = value

        def __eq__(self, other):
            if self._box == other._box and self._depth == other._depth:
                return True
            return False

        def __str__(self):
            return str(_to_internal_settings_region_of_interest(self))

    class Sampling:

        class Color:

            disabled = "disabled"
            grayscale = "grayscale"
            rgb = "rgb"

            _valid_values = {
                "disabled": _zivid.Settings.Sampling.Color.disabled,
                "grayscale": _zivid.Settings.Sampling.Color.grayscale,
                "rgb": _zivid.Settings.Sampling.Color.rgb,
            }

            @classmethod
            def valid_values(cls):
                return list(cls._valid_values.keys())

        class Pixel:

            all = "all"
            blueSubsample2x2 = "blueSubsample2x2"
            blueSubsample4x4 = "blueSubsample4x4"
            by2x2 = "by2x2"
            by4x4 = "by4x4"
            redSubsample2x2 = "redSubsample2x2"
            redSubsample4x4 = "redSubsample4x4"

            _valid_values = {
                "all": _zivid.Settings.Sampling.Pixel.all,
                "blueSubsample2x2": _zivid.Settings.Sampling.Pixel.blueSubsample2x2,
                "blueSubsample4x4": _zivid.Settings.Sampling.Pixel.blueSubsample4x4,
                "by2x2": _zivid.Settings.Sampling.Pixel.by2x2,
                "by4x4": _zivid.Settings.Sampling.Pixel.by4x4,
                "redSubsample2x2": _zivid.Settings.Sampling.Pixel.redSubsample2x2,
                "redSubsample4x4": _zivid.Settings.Sampling.Pixel.redSubsample4x4,
            }

            @classmethod
            def valid_values(cls):
                return list(cls._valid_values.keys())

        def __init__(
            self,
            color=_zivid.Settings.Sampling.Color().value,
            pixel=_zivid.Settings.Sampling.Pixel().value,
        ):

            if isinstance(color, _zivid.Settings.Sampling.Color.enum) or color is None:
                self._color = _zivid.Settings.Sampling.Color(color)
            elif isinstance(color, str):
                self._color = _zivid.Settings.Sampling.Color(
                    self.Color._valid_values[color]
                )
            else:
                raise TypeError(
                    "Unsupported type, expected: str or None, got {value_type}".format(
                        value_type=type(color)
                    )
                )

            if isinstance(pixel, _zivid.Settings.Sampling.Pixel.enum) or pixel is None:
                self._pixel = _zivid.Settings.Sampling.Pixel(pixel)
            elif isinstance(pixel, str):
                self._pixel = _zivid.Settings.Sampling.Pixel(
                    self.Pixel._valid_values[pixel]
                )
            else:
                raise TypeError(
                    "Unsupported type, expected: str or None, got {value_type}".format(
                        value_type=type(pixel)
                    )
                )

        @property
        def color(self):
            if self._color.value is None:
                return None
            for key, internal_value in self.Color._valid_values.items():
                if internal_value == self._color.value:
                    return key
            raise ValueError("Unsupported value {value}".format(value=self._color))

        @property
        def pixel(self):
            if self._pixel.value is None:
                return None
            for key, internal_value in self.Pixel._valid_values.items():
                if internal_value == self._pixel.value:
                    return key
            raise ValueError("Unsupported value {value}".format(value=self._pixel))

        @color.setter
        def color(self, value):
            if isinstance(value, str):
                self._color = _zivid.Settings.Sampling.Color(
                    self.Color._valid_values[value]
                )
            elif (
                isinstance(value, _zivid.Settings.Sampling.Color.enum) or value is None
            ):
                self._color = _zivid.Settings.Sampling.Color(value)
            else:
                raise TypeError(
                    "Unsupported type, expected: str or None, got {value_type}".format(
                        value_type=type(value)
                    )
                )

        @pixel.setter
        def pixel(self, value):
            if isinstance(value, str):
                self._pixel = _zivid.Settings.Sampling.Pixel(
                    self.Pixel._valid_values[value]
                )
            elif (
                isinstance(value, _zivid.Settings.Sampling.Pixel.enum) or value is None
            ):
                self._pixel = _zivid.Settings.Sampling.Pixel(value)
            else:
                raise TypeError(
                    "Unsupported type, expected: str or None, got {value_type}".format(
                        value_type=type(value)
                    )
                )

        def __eq__(self, other):
            if self._color == other._color and self._pixel == other._pixel:
                return True
            return False

        def __str__(self):
            return str(_to_internal_settings_sampling(self))

    class Engine:

        omni = "omni"
        phase = "phase"
        sage = "sage"
        stripe = "stripe"

        _valid_values = {
            "omni": _zivid.Settings.Engine.omni,
            "phase": _zivid.Settings.Engine.phase,
            "sage": _zivid.Settings.Engine.sage,
            "stripe": _zivid.Settings.Engine.stripe,
        }

        @classmethod
        def valid_values(cls):
            return list(cls._valid_values.keys())

    def __init__(
        self,
        acquisitions=None,
        color=None,
        engine=_zivid.Settings.Engine().value,
        diagnostics=None,
        processing=None,
        region_of_interest=None,
        sampling=None,
    ):

        if acquisitions is None:
            self._acquisitions = []
        elif isinstance(acquisitions, (collections.abc.Iterable,)):
            self._acquisitions = []
            for item in acquisitions:
                if isinstance(item, self.Acquisition):
                    self._acquisitions.append(item)
                else:
                    raise TypeError(
                        "Unsupported type {item_type}".format(item_type=type(item))
                    )
        else:
            raise TypeError(
                "Unsupported type, expected: (collections.abc.Iterable,) or None, got {value_type}".format(
                    value_type=type(acquisitions)
                )
            )

        if isinstance(color, zivid.settings2d.Settings2D) or color is None:
            self._color = color
        else:
            raise TypeError(
                "Unsupported type, expected: zivid.settings2d.Settings2D or None, got {value_type}".format(
                    value_type=type(color)
                )
            )

        if isinstance(engine, _zivid.Settings.Engine.enum) or engine is None:
            self._engine = _zivid.Settings.Engine(engine)
        elif isinstance(engine, str):
            self._engine = _zivid.Settings.Engine(self.Engine._valid_values[engine])
        else:
            raise TypeError(
                "Unsupported type, expected: str or None, got {value_type}".format(
                    value_type=type(engine)
                )
            )

        if diagnostics is None:
            diagnostics = self.Diagnostics()
        if not isinstance(diagnostics, self.Diagnostics):
            raise TypeError("Unsupported type: {value}".format(value=type(diagnostics)))
        self._diagnostics = diagnostics

        if processing is None:
            processing = self.Processing()
        if not isinstance(processing, self.Processing):
            raise TypeError("Unsupported type: {value}".format(value=type(processing)))
        self._processing = processing

        if region_of_interest is None:
            region_of_interest = self.RegionOfInterest()
        if not isinstance(region_of_interest, self.RegionOfInterest):
            raise TypeError(
                "Unsupported type: {value}".format(value=type(region_of_interest))
            )
        self._region_of_interest = region_of_interest

        if sampling is None:
            sampling = self.Sampling()
        if not isinstance(sampling, self.Sampling):
            raise TypeError("Unsupported type: {value}".format(value=type(sampling)))
        self._sampling = sampling

    @property
    def acquisitions(self):
        return self._acquisitions

    @property
    def color(self):
        return self._color

    @property
    def engine(self):
        if self._engine.value is None:
            return None
        for key, internal_value in self.Engine._valid_values.items():
            if internal_value == self._engine.value:
                return key
        raise ValueError("Unsupported value {value}".format(value=self._engine))

    @property
    def diagnostics(self):
        return self._diagnostics

    @property
    def processing(self):
        return self._processing

    @property
    def region_of_interest(self):
        return self._region_of_interest

    @property
    def sampling(self):
        return self._sampling

    @acquisitions.setter
    def acquisitions(self, value):
        if not isinstance(value, (collections.abc.Iterable,)):
            raise TypeError("Unsupported type {value}".format(value=type(value)))
        self._acquisitions = []
        for item in value:
            if isinstance(item, self.Acquisition):
                self._acquisitions.append(item)
            else:
                raise TypeError(
                    "Unsupported type {item_type}".format(item_type=type(item))
                )

    @color.setter
    def color(self, value):
        if isinstance(value, zivid.settings2d.Settings2D) or value is None:
            self._color = value
        else:
            raise TypeError(
                "Unsupported type, expected: zivid.settings2d.Settings2D or None, got {value_type}".format(
                    value_type=type(value)
                )
            )

    @engine.setter
    def engine(self, value):
        if isinstance(value, str):
            self._engine = _zivid.Settings.Engine(self.Engine._valid_values[value])
        elif isinstance(value, _zivid.Settings.Engine.enum) or value is None:
            self._engine = _zivid.Settings.Engine(value)
        else:
            raise TypeError(
                "Unsupported type, expected: str or None, got {value_type}".format(
                    value_type=type(value)
                )
            )

    @diagnostics.setter
    def diagnostics(self, value):
        if not isinstance(value, self.Diagnostics):
            raise TypeError("Unsupported type {value}".format(value=type(value)))
        self._diagnostics = value

    @processing.setter
    def processing(self, value):
        if not isinstance(value, self.Processing):
            raise TypeError("Unsupported type {value}".format(value=type(value)))
        self._processing = value

    @region_of_interest.setter
    def region_of_interest(self, value):
        if not isinstance(value, self.RegionOfInterest):
            raise TypeError("Unsupported type {value}".format(value=type(value)))
        self._region_of_interest = value

    @sampling.setter
    def sampling(self, value):
        if not isinstance(value, self.Sampling):
            raise TypeError("Unsupported type {value}".format(value=type(value)))
        self._sampling = value

    @classmethod
    def load(cls, file_name):
        return _to_settings(_zivid.Settings(str(file_name)))

    def save(self, file_name):
        _to_internal_settings(self).save(str(file_name))

    @classmethod
    def from_serialized(cls, value):
        return _to_settings(_zivid.Settings.from_serialized(str(value)))

    def serialize(self):
        return _to_internal_settings(self).serialize()

    def __eq__(self, other):
        if (
            self._acquisitions == other._acquisitions
            and self._color == other._color
            and self._engine == other._engine
            and self._diagnostics == other._diagnostics
            and self._processing == other._processing
            and self._region_of_interest == other._region_of_interest
            and self._sampling == other._sampling
        ):
            return True
        return False

    def __str__(self):
        return str(_to_internal_settings(self))


def _to_settings_acquisition(internal_acquisition):
    return Settings.Acquisition(
        aperture=internal_acquisition.aperture.value,
        brightness=internal_acquisition.brightness.value,
        exposure_time=internal_acquisition.exposure_time.value,
        gain=internal_acquisition.gain.value,
    )


def _to_settings_diagnostics(internal_diagnostics):
    return Settings.Diagnostics(
        enabled=internal_diagnostics.enabled.value,
    )


def _to_settings_processing_color_balance(internal_balance):
    return Settings.Processing.Color.Balance(
        blue=internal_balance.blue.value,
        green=internal_balance.green.value,
        red=internal_balance.red.value,
    )


def _to_settings_processing_color_experimental(internal_experimental):
    return Settings.Processing.Color.Experimental(
        mode=internal_experimental.mode.value,
    )


def _to_settings_processing_color(internal_color):
    return Settings.Processing.Color(
        balance=_to_settings_processing_color_balance(internal_color.balance),
        experimental=_to_settings_processing_color_experimental(
            internal_color.experimental
        ),
        gamma=internal_color.gamma.value,
    )


def _to_settings_processing_filters_cluster_removal(internal_removal):
    return Settings.Processing.Filters.Cluster.Removal(
        enabled=internal_removal.enabled.value,
        max_neighbor_distance=internal_removal.max_neighbor_distance.value,
        min_area=internal_removal.min_area.value,
    )


def _to_settings_processing_filters_cluster(internal_cluster):
    return Settings.Processing.Filters.Cluster(
        removal=_to_settings_processing_filters_cluster_removal(
            internal_cluster.removal
        ),
    )


def _to_settings_processing_filters_experimental_contrast_distortion_correction(
    internal_correction,
):
    return Settings.Processing.Filters.Experimental.ContrastDistortion.Correction(
        enabled=internal_correction.enabled.value,
        strength=internal_correction.strength.value,
    )


def _to_settings_processing_filters_experimental_contrast_distortion_removal(
    internal_removal,
):
    return Settings.Processing.Filters.Experimental.ContrastDistortion.Removal(
        enabled=internal_removal.enabled.value,
        threshold=internal_removal.threshold.value,
    )


def _to_settings_processing_filters_experimental_contrast_distortion(
    internal_contrast_distortion,
):
    return Settings.Processing.Filters.Experimental.ContrastDistortion(
        correction=_to_settings_processing_filters_experimental_contrast_distortion_correction(
            internal_contrast_distortion.correction
        ),
        removal=_to_settings_processing_filters_experimental_contrast_distortion_removal(
            internal_contrast_distortion.removal
        ),
    )


def _to_settings_processing_filters_experimental(internal_experimental):
    return Settings.Processing.Filters.Experimental(
        contrast_distortion=_to_settings_processing_filters_experimental_contrast_distortion(
            internal_experimental.contrast_distortion
        ),
    )


def _to_settings_processing_filters_hole_repair(internal_repair):
    return Settings.Processing.Filters.Hole.Repair(
        enabled=internal_repair.enabled.value,
        hole_size=internal_repair.hole_size.value,
        strictness=internal_repair.strictness.value,
    )


def _to_settings_processing_filters_hole(internal_hole):
    return Settings.Processing.Filters.Hole(
        repair=_to_settings_processing_filters_hole_repair(internal_hole.repair),
    )


def _to_settings_processing_filters_noise_removal(internal_removal):
    return Settings.Processing.Filters.Noise.Removal(
        enabled=internal_removal.enabled.value,
        threshold=internal_removal.threshold.value,
    )


def _to_settings_processing_filters_noise_repair(internal_repair):
    return Settings.Processing.Filters.Noise.Repair(
        enabled=internal_repair.enabled.value,
    )


def _to_settings_processing_filters_noise_suppression(internal_suppression):
    return Settings.Processing.Filters.Noise.Suppression(
        enabled=internal_suppression.enabled.value,
    )


def _to_settings_processing_filters_noise(internal_noise):
    return Settings.Processing.Filters.Noise(
        removal=_to_settings_processing_filters_noise_removal(internal_noise.removal),
        repair=_to_settings_processing_filters_noise_repair(internal_noise.repair),
        suppression=_to_settings_processing_filters_noise_suppression(
            internal_noise.suppression
        ),
    )


def _to_settings_processing_filters_outlier_removal(internal_removal):
    return Settings.Processing.Filters.Outlier.Removal(
        enabled=internal_removal.enabled.value,
        threshold=internal_removal.threshold.value,
    )


def _to_settings_processing_filters_outlier(internal_outlier):
    return Settings.Processing.Filters.Outlier(
        removal=_to_settings_processing_filters_outlier_removal(
            internal_outlier.removal
        ),
    )


def _to_settings_processing_filters_reflection_removal(internal_removal):
    return Settings.Processing.Filters.Reflection.Removal(
        enabled=internal_removal.enabled.value,
        mode=internal_removal.mode.value,
    )


def _to_settings_processing_filters_reflection(internal_reflection):
    return Settings.Processing.Filters.Reflection(
        removal=_to_settings_processing_filters_reflection_removal(
            internal_reflection.removal
        ),
    )


def _to_settings_processing_filters_smoothing_gaussian(internal_gaussian):
    return Settings.Processing.Filters.Smoothing.Gaussian(
        enabled=internal_gaussian.enabled.value,
        sigma=internal_gaussian.sigma.value,
    )


def _to_settings_processing_filters_smoothing(internal_smoothing):
    return Settings.Processing.Filters.Smoothing(
        gaussian=_to_settings_processing_filters_smoothing_gaussian(
            internal_smoothing.gaussian
        ),
    )


def _to_settings_processing_filters(internal_filters):
    return Settings.Processing.Filters(
        cluster=_to_settings_processing_filters_cluster(internal_filters.cluster),
        experimental=_to_settings_processing_filters_experimental(
            internal_filters.experimental
        ),
        hole=_to_settings_processing_filters_hole(internal_filters.hole),
        noise=_to_settings_processing_filters_noise(internal_filters.noise),
        outlier=_to_settings_processing_filters_outlier(internal_filters.outlier),
        reflection=_to_settings_processing_filters_reflection(
            internal_filters.reflection
        ),
        smoothing=_to_settings_processing_filters_smoothing(internal_filters.smoothing),
    )


def _to_settings_processing_resampling(internal_resampling):
    return Settings.Processing.Resampling(
        mode=internal_resampling.mode.value,
    )


def _to_settings_processing(internal_processing):
    return Settings.Processing(
        color=_to_settings_processing_color(internal_processing.color),
        filters=_to_settings_processing_filters(internal_processing.filters),
        resampling=_to_settings_processing_resampling(internal_processing.resampling),
    )


def _to_settings_region_of_interest_box(internal_box):
    return Settings.RegionOfInterest.Box(
        enabled=internal_box.enabled.value,
        extents=internal_box.extents.value,
        point_a=internal_box.point_a.value,
        point_b=internal_box.point_b.value,
        point_o=internal_box.point_o.value,
    )


def _to_settings_region_of_interest_depth(internal_depth):
    return Settings.RegionOfInterest.Depth(
        enabled=internal_depth.enabled.value,
        range=internal_depth.range.value,
    )


def _to_settings_region_of_interest(internal_region_of_interest):
    return Settings.RegionOfInterest(
        box=_to_settings_region_of_interest_box(internal_region_of_interest.box),
        depth=_to_settings_region_of_interest_depth(internal_region_of_interest.depth),
    )


def _to_settings_sampling(internal_sampling):
    return Settings.Sampling(
        color=internal_sampling.color.value,
        pixel=internal_sampling.pixel.value,
    )


def _to_settings(internal_settings):
    return Settings(
        acquisitions=[
            _to_settings_acquisition(value)
            for value in internal_settings.acquisitions.value
        ],
        diagnostics=_to_settings_diagnostics(internal_settings.diagnostics),
        processing=_to_settings_processing(internal_settings.processing),
        region_of_interest=_to_settings_region_of_interest(
            internal_settings.region_of_interest
        ),
        sampling=_to_settings_sampling(internal_settings.sampling),
        color=(
            zivid.settings2d._to_settings2d(internal_settings.color.value)
            if internal_settings.color.value is not None
            else None
        ),
        engine=internal_settings.engine.value,
    )


def _to_internal_settings_acquisition(acquisition):
    internal_acquisition = _zivid.Settings.Acquisition()

    internal_acquisition.aperture = _zivid.Settings.Acquisition.Aperture(
        acquisition.aperture
    )
    internal_acquisition.brightness = _zivid.Settings.Acquisition.Brightness(
        acquisition.brightness
    )
    internal_acquisition.exposure_time = _zivid.Settings.Acquisition.ExposureTime(
        acquisition.exposure_time
    )
    internal_acquisition.gain = _zivid.Settings.Acquisition.Gain(acquisition.gain)

    return internal_acquisition


def _to_internal_settings_diagnostics(diagnostics):
    internal_diagnostics = _zivid.Settings.Diagnostics()

    internal_diagnostics.enabled = _zivid.Settings.Diagnostics.Enabled(
        diagnostics.enabled
    )

    return internal_diagnostics


def _to_internal_settings_processing_color_balance(balance):
    internal_balance = _zivid.Settings.Processing.Color.Balance()

    internal_balance.blue = _zivid.Settings.Processing.Color.Balance.Blue(balance.blue)
    internal_balance.green = _zivid.Settings.Processing.Color.Balance.Green(
        balance.green
    )
    internal_balance.red = _zivid.Settings.Processing.Color.Balance.Red(balance.red)

    return internal_balance


def _to_internal_settings_processing_color_experimental(experimental):
    internal_experimental = _zivid.Settings.Processing.Color.Experimental()

    internal_experimental.mode = _zivid.Settings.Processing.Color.Experimental.Mode(
        experimental._mode.value
    )

    return internal_experimental


def _to_internal_settings_processing_color(color):
    internal_color = _zivid.Settings.Processing.Color()

    internal_color.gamma = _zivid.Settings.Processing.Color.Gamma(color.gamma)

    internal_color.balance = _to_internal_settings_processing_color_balance(
        color.balance
    )
    internal_color.experimental = _to_internal_settings_processing_color_experimental(
        color.experimental
    )
    return internal_color


def _to_internal_settings_processing_filters_cluster_removal(removal):
    internal_removal = _zivid.Settings.Processing.Filters.Cluster.Removal()

    internal_removal.enabled = (
        _zivid.Settings.Processing.Filters.Cluster.Removal.Enabled(removal.enabled)
    )
    internal_removal.max_neighbor_distance = (
        _zivid.Settings.Processing.Filters.Cluster.Removal.MaxNeighborDistance(
            removal.max_neighbor_distance
        )
    )
    internal_removal.min_area = (
        _zivid.Settings.Processing.Filters.Cluster.Removal.MinArea(removal.min_area)
    )

    return internal_removal


def _to_internal_settings_processing_filters_cluster(cluster):
    internal_cluster = _zivid.Settings.Processing.Filters.Cluster()

    internal_cluster.removal = _to_internal_settings_processing_filters_cluster_removal(
        cluster.removal
    )
    return internal_cluster


def _to_internal_settings_processing_filters_experimental_contrast_distortion_correction(
    correction,
):
    internal_correction = (
        _zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Correction()
    )

    internal_correction.enabled = _zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Correction.Enabled(
        correction.enabled
    )
    internal_correction.strength = _zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Correction.Strength(
        correction.strength
    )

    return internal_correction


def _to_internal_settings_processing_filters_experimental_contrast_distortion_removal(
    removal,
):
    internal_removal = (
        _zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Removal()
    )

    internal_removal.enabled = _zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Removal.Enabled(
        removal.enabled
    )
    internal_removal.threshold = _zivid.Settings.Processing.Filters.Experimental.ContrastDistortion.Removal.Threshold(
        removal.threshold
    )

    return internal_removal


def _to_internal_settings_processing_filters_experimental_contrast_distortion(
    contrast_distortion,
):
    internal_contrast_distortion = (
        _zivid.Settings.Processing.Filters.Experimental.ContrastDistortion()
    )

    internal_contrast_distortion.correction = _to_internal_settings_processing_filters_experimental_contrast_distortion_correction(
        contrast_distortion.correction
    )
    internal_contrast_distortion.removal = _to_internal_settings_processing_filters_experimental_contrast_distortion_removal(
        contrast_distortion.removal
    )
    return internal_contrast_distortion


def _to_internal_settings_processing_filters_experimental(experimental):
    internal_experimental = _zivid.Settings.Processing.Filters.Experimental()

    internal_experimental.contrast_distortion = (
        _to_internal_settings_processing_filters_experimental_contrast_distortion(
            experimental.contrast_distortion
        )
    )
    return internal_experimental


def _to_internal_settings_processing_filters_hole_repair(repair):
    internal_repair = _zivid.Settings.Processing.Filters.Hole.Repair()

    internal_repair.enabled = _zivid.Settings.Processing.Filters.Hole.Repair.Enabled(
        repair.enabled
    )
    internal_repair.hole_size = _zivid.Settings.Processing.Filters.Hole.Repair.HoleSize(
        repair.hole_size
    )
    internal_repair.strictness = (
        _zivid.Settings.Processing.Filters.Hole.Repair.Strictness(repair.strictness)
    )

    return internal_repair


def _to_internal_settings_processing_filters_hole(hole):
    internal_hole = _zivid.Settings.Processing.Filters.Hole()

    internal_hole.repair = _to_internal_settings_processing_filters_hole_repair(
        hole.repair
    )
    return internal_hole


def _to_internal_settings_processing_filters_noise_removal(removal):
    internal_removal = _zivid.Settings.Processing.Filters.Noise.Removal()

    internal_removal.enabled = _zivid.Settings.Processing.Filters.Noise.Removal.Enabled(
        removal.enabled
    )
    internal_removal.threshold = (
        _zivid.Settings.Processing.Filters.Noise.Removal.Threshold(removal.threshold)
    )

    return internal_removal


def _to_internal_settings_processing_filters_noise_repair(repair):
    internal_repair = _zivid.Settings.Processing.Filters.Noise.Repair()

    internal_repair.enabled = _zivid.Settings.Processing.Filters.Noise.Repair.Enabled(
        repair.enabled
    )

    return internal_repair


def _to_internal_settings_processing_filters_noise_suppression(suppression):
    internal_suppression = _zivid.Settings.Processing.Filters.Noise.Suppression()

    internal_suppression.enabled = (
        _zivid.Settings.Processing.Filters.Noise.Suppression.Enabled(
            suppression.enabled
        )
    )

    return internal_suppression


def _to_internal_settings_processing_filters_noise(noise):
    internal_noise = _zivid.Settings.Processing.Filters.Noise()

    internal_noise.removal = _to_internal_settings_processing_filters_noise_removal(
        noise.removal
    )
    internal_noise.repair = _to_internal_settings_processing_filters_noise_repair(
        noise.repair
    )
    internal_noise.suppression = (
        _to_internal_settings_processing_filters_noise_suppression(noise.suppression)
    )
    return internal_noise


def _to_internal_settings_processing_filters_outlier_removal(removal):
    internal_removal = _zivid.Settings.Processing.Filters.Outlier.Removal()

    internal_removal.enabled = (
        _zivid.Settings.Processing.Filters.Outlier.Removal.Enabled(removal.enabled)
    )
    internal_removal.threshold = (
        _zivid.Settings.Processing.Filters.Outlier.Removal.Threshold(removal.threshold)
    )

    return internal_removal


def _to_internal_settings_processing_filters_outlier(outlier):
    internal_outlier = _zivid.Settings.Processing.Filters.Outlier()

    internal_outlier.removal = _to_internal_settings_processing_filters_outlier_removal(
        outlier.removal
    )
    return internal_outlier


def _to_internal_settings_processing_filters_reflection_removal(removal):
    internal_removal = _zivid.Settings.Processing.Filters.Reflection.Removal()

    internal_removal.enabled = (
        _zivid.Settings.Processing.Filters.Reflection.Removal.Enabled(removal.enabled)
    )
    internal_removal.mode = _zivid.Settings.Processing.Filters.Reflection.Removal.Mode(
        removal._mode.value
    )

    return internal_removal


def _to_internal_settings_processing_filters_reflection(reflection):
    internal_reflection = _zivid.Settings.Processing.Filters.Reflection()

    internal_reflection.removal = (
        _to_internal_settings_processing_filters_reflection_removal(reflection.removal)
    )
    return internal_reflection


def _to_internal_settings_processing_filters_smoothing_gaussian(gaussian):
    internal_gaussian = _zivid.Settings.Processing.Filters.Smoothing.Gaussian()

    internal_gaussian.enabled = (
        _zivid.Settings.Processing.Filters.Smoothing.Gaussian.Enabled(gaussian.enabled)
    )
    internal_gaussian.sigma = (
        _zivid.Settings.Processing.Filters.Smoothing.Gaussian.Sigma(gaussian.sigma)
    )

    return internal_gaussian


def _to_internal_settings_processing_filters_smoothing(smoothing):
    internal_smoothing = _zivid.Settings.Processing.Filters.Smoothing()

    internal_smoothing.gaussian = (
        _to_internal_settings_processing_filters_smoothing_gaussian(smoothing.gaussian)
    )
    return internal_smoothing


def _to_internal_settings_processing_filters(filters):
    internal_filters = _zivid.Settings.Processing.Filters()

    internal_filters.cluster = _to_internal_settings_processing_filters_cluster(
        filters.cluster
    )
    internal_filters.experimental = (
        _to_internal_settings_processing_filters_experimental(filters.experimental)
    )
    internal_filters.hole = _to_internal_settings_processing_filters_hole(filters.hole)
    internal_filters.noise = _to_internal_settings_processing_filters_noise(
        filters.noise
    )
    internal_filters.outlier = _to_internal_settings_processing_filters_outlier(
        filters.outlier
    )
    internal_filters.reflection = _to_internal_settings_processing_filters_reflection(
        filters.reflection
    )
    internal_filters.smoothing = _to_internal_settings_processing_filters_smoothing(
        filters.smoothing
    )
    return internal_filters


def _to_internal_settings_processing_resampling(resampling):
    internal_resampling = _zivid.Settings.Processing.Resampling()

    internal_resampling.mode = _zivid.Settings.Processing.Resampling.Mode(
        resampling._mode.value
    )

    return internal_resampling


def _to_internal_settings_processing(processing):
    internal_processing = _zivid.Settings.Processing()

    internal_processing.color = _to_internal_settings_processing_color(processing.color)
    internal_processing.filters = _to_internal_settings_processing_filters(
        processing.filters
    )
    internal_processing.resampling = _to_internal_settings_processing_resampling(
        processing.resampling
    )
    return internal_processing


def _to_internal_settings_region_of_interest_box(box):
    internal_box = _zivid.Settings.RegionOfInterest.Box()

    internal_box.enabled = _zivid.Settings.RegionOfInterest.Box.Enabled(box.enabled)
    internal_box.extents = _zivid.Settings.RegionOfInterest.Box.Extents(box.extents)
    internal_box.point_a = _zivid.Settings.RegionOfInterest.Box.PointA(box.point_a)
    internal_box.point_b = _zivid.Settings.RegionOfInterest.Box.PointB(box.point_b)
    internal_box.point_o = _zivid.Settings.RegionOfInterest.Box.PointO(box.point_o)

    return internal_box


def _to_internal_settings_region_of_interest_depth(depth):
    internal_depth = _zivid.Settings.RegionOfInterest.Depth()

    internal_depth.enabled = _zivid.Settings.RegionOfInterest.Depth.Enabled(
        depth.enabled
    )
    internal_depth.range = _zivid.Settings.RegionOfInterest.Depth.Range(depth.range)

    return internal_depth


def _to_internal_settings_region_of_interest(region_of_interest):
    internal_region_of_interest = _zivid.Settings.RegionOfInterest()

    internal_region_of_interest.box = _to_internal_settings_region_of_interest_box(
        region_of_interest.box
    )
    internal_region_of_interest.depth = _to_internal_settings_region_of_interest_depth(
        region_of_interest.depth
    )
    return internal_region_of_interest


def _to_internal_settings_sampling(sampling):
    internal_sampling = _zivid.Settings.Sampling()

    internal_sampling.color = _zivid.Settings.Sampling.Color(sampling._color.value)
    internal_sampling.pixel = _zivid.Settings.Sampling.Pixel(sampling._pixel.value)

    return internal_sampling


def _to_internal_settings(settings):
    internal_settings = _zivid.Settings()

    temp_acquisitions = _zivid.Settings.Acquisitions()
    for value in settings.acquisitions:
        temp_acquisitions.append(_to_internal_settings_acquisition(value))
    internal_settings.acquisitions = temp_acquisitions

    internal_settings.color = _zivid.Settings.Color(
        zivid.settings2d._to_internal_settings2d(settings.color)
        if settings.color is not None
        else None
    )
    internal_settings.engine = _zivid.Settings.Engine(settings._engine.value)

    internal_settings.diagnostics = _to_internal_settings_diagnostics(
        settings.diagnostics
    )
    internal_settings.processing = _to_internal_settings_processing(settings.processing)
    internal_settings.region_of_interest = _to_internal_settings_region_of_interest(
        settings.region_of_interest
    )
    internal_settings.sampling = _to_internal_settings_sampling(settings.sampling)
    return internal_settings



"""Contains the Frame class."""

import _zivid
from zivid.settings2d import _to_settings2d
from zivid.camera_info import _to_camera_info
from zivid.camera_state import _to_camera_state
from zivid.frame_info import _to_frame_info
from zivid.image import Image


class Frame2D:
    """A 2D frame captured by a Zivid camera.

    Contains a 2D image as well as metadata, settings and state of the API at the time of capture.
    """

    def __init__(self, impl):
        """Initialize Frame2D wrapper.

        This constructor is only used internally, and should not be called by the end-user.

        Args:
            impl:   Reference to internal/back-end instance.

        Raises:
            TypeError: If argument does not match the expected internal class.
        """
        if isinstance(impl, _zivid.Frame2D):
            self.__impl = impl
        else:
            raise TypeError(
                "Unsupported type for argument impl. Got {}, expected {}.".format(
                    type(impl), _zivid.Frame2D
                )
            )

    def __str__(self):
        return str(self.__impl)

    def image_rgba(self):
        """Get color (RGBA) image from the frame.

        Returns:
            An image instance containing RGBA data
        """
        return Image(self.__impl.image_rgba())

    def image_bgra(self):
        """Get color (BGRA) image from the frame.

        Returns:
            An image instance containing BGRA data
        """
        return Image(self.__impl.image_bgra())

    def image_srgb(self):
        """Get color (RGBA) image from the frame in the sRGB color space.

        Returns:
            An image instance containing RGBA data in sRGB color space
        """
        return Image(self.__impl.image_srgb())

    @property
    def settings(self):
        """Get the settings used to capture this frame.

        Returns:
            A Settings2D instance
        """
        return _to_settings2d(self.__impl.settings)

    @property
    def state(self):
        """Get the camera state data at the time of the frame capture.

        Returns:
            A CameraState instance
        """
        return _to_camera_state(self.__impl.state)

    @property
    def info(self):
        """Get information collected at the time of the capture.

        Returns:
            A FrameInfo instance
        """
        return _to_frame_info(self.__impl.info)

    @property
    def camera_info(self):
        """Get information about the camera used to capture the frame.

        Returns:
            A CameraInfo instance
        """
        return _to_camera_info(self.__impl.camera_info)

    def release(self):
        """Release the underlying resources."""
        try:
            impl = self.__impl
        except AttributeError:
            pass
        else:
            impl.release()

    def __enter__(self):
        return self

    def __exit__(self, exception_type, exception_value, traceback):
        self.release()

    def __del__(self):
        self.release()



import numpy as np
import pytest


def test_image_context_manager(frame_2d):
    import zivid

    with frame_2d.image_rgba() as image_rgba:
        assert image_rgba is not None
        assert isinstance(image_rgba, zivid.Image)

    with frame_2d.image_bgra() as image_bgra:
        assert image_bgra is not None
        assert isinstance(image_bgra, zivid.Image)


def test_image(frame_2d):
    import zivid

    image_rgba = frame_2d.image_rgba()
    assert image_rgba is not None
    assert isinstance(image_rgba, zivid.Image)

    image_bgra = frame_2d.image_bgra()
    assert image_bgra is not None
    assert isinstance(image_bgra, zivid.Image)


def test_image_rgba_bgra_correspondence(frame_2d):
    rgba = frame_2d.image_rgba().copy_data()
    bgra = frame_2d.image_bgra().copy_data()

    np.testing.assert_array_equal(bgra[:, :, 0], rgba[:, :, 2])
    np.testing.assert_array_equal(bgra[:, :, 1], rgba[:, :, 1])
    np.testing.assert_array_equal(bgra[:, :, 2], rgba[:, :, 0])
    np.testing.assert_array_equal(bgra[:, :, 3], rgba[:, :, 3])


def test_state(frame_2d):
    import zivid

    state = frame_2d.state
    assert state is not None
    assert isinstance(state, zivid.CameraState)


def test_info(frame_2d):
    import zivid

    info = frame_2d.info
    assert info is not None
    assert isinstance(info, zivid.FrameInfo)


def test_camera_info(frame_2d):
    from zivid.camera_info import CameraInfo

    camera_info = frame_2d.camera_info
    assert camera_info
    assert isinstance(camera_info, CameraInfo)


def test_settings(frame_2d):
    import zivid

    settings_2d = frame_2d.settings
    assert settings_2d is not None
    assert isinstance(settings_2d, zivid.Settings2D)


def test_release(frame_2d):
    frame_2d.image_rgba()
    frame_2d.release()
    with pytest.raises(RuntimeError):
        frame_2d.image_rgba()


def test_context_manager(shared_file_camera):
    import zivid

    settings_2d = zivid.Settings2D(acquisitions=[zivid.Settings2D.Acquisition()])
    with shared_file_camera.capture(settings_2d) as frame_2d:
        frame_2d.image_rgba()
    with pytest.raises(RuntimeError):
        frame_2d.image_rgba()

    with shared_file_camera.capture(settings_2d) as frame_2d:
        frame_2d.image_bgra()
    with pytest.raises(RuntimeError):
        frame_2d.image_bgra()



"""Auto generated, do not edit."""

# pylint: disable=too-many-lines,protected-access,too-few-public-methods,too-many-arguments,line-too-long,missing-function-docstring,missing-class-docstring,redefined-builtin,too-many-branches,too-many-boolean-expressions
import collections.abc
import _zivid


class CameraState:

    class Network:

        class LocalInterface:

            class IPV4:

                class Subnet:

                    def __init__(
                        self,
                        address=_zivid.CameraState.Network.LocalInterface.IPV4.Subnet.Address().value,
                        mask=_zivid.CameraState.Network.LocalInterface.IPV4.Subnet.Mask().value,
                    ):

                        if isinstance(address, (str,)):
                            self._address = _zivid.CameraState.Network.LocalInterface.IPV4.Subnet.Address(
                                address
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: (str,), got {value_type}".format(
                                    value_type=type(address)
                                )
                            )

                        if isinstance(mask, (str,)):
                            self._mask = _zivid.CameraState.Network.LocalInterface.IPV4.Subnet.Mask(
                                mask
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: (str,), got {value_type}".format(
                                    value_type=type(mask)
                                )
                            )

                    @property
                    def address(self):
                        return self._address.value

                    @property
                    def mask(self):
                        return self._mask.value

                    @address.setter
                    def address(self, value):
                        if isinstance(value, (str,)):
                            self._address = _zivid.CameraState.Network.LocalInterface.IPV4.Subnet.Address(
                                value
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: str, got {value_type}".format(
                                    value_type=type(value)
                                )
                            )

                    @mask.setter
                    def mask(self, value):
                        if isinstance(value, (str,)):
                            self._mask = _zivid.CameraState.Network.LocalInterface.IPV4.Subnet.Mask(
                                value
                            )
                        else:
                            raise TypeError(
                                "Unsupported type, expected: str, got {value_type}".format(
                                    value_type=type(value)
                                )
                            )

                    def __eq__(self, other):
                        if (
                            self._address == other._address
                            and self._mask == other._mask
                        ):
                            return True
                        return False

                    def __str__(self):
                        return str(
                            _to_internal_camera_state_network_local_interface_ipv4_subnet(
                                self
                            )
                        )

                def __init__(
                    self,
                    subnets=None,
                ):

                    if subnets is None:
                        self._subnets = []
                    elif isinstance(subnets, (collections.abc.Iterable,)):
                        self._subnets = []
                        for item in subnets:
                            if isinstance(item, self.Subnet):
                                self._subnets.append(item)
                            else:
                                raise TypeError(
                                    "Unsupported type {item_type}".format(
                                        item_type=type(item)
                                    )
                                )
                    else:
                        raise TypeError(
                            "Unsupported type, expected: (collections.abc.Iterable,) or None, got {value_type}".format(
                                value_type=type(subnets)
                            )
                        )

                @property
                def subnets(self):
                    return self._subnets

                @subnets.setter
                def subnets(self, value):
                    if not isinstance(value, (collections.abc.Iterable,)):
                        raise TypeError(
                            "Unsupported type {value}".format(value=type(value))
                        )
                    self._subnets = []
                    for item in value:
                        if isinstance(item, self.Subnet):
                            self._subnets.append(item)
                        else:
                            raise TypeError(
                                "Unsupported type {item_type}".format(
                                    item_type=type(item)
                                )
                            )

                def __eq__(self, other):
                    if self._subnets == other._subnets:
                        return True
                    return False

                def __str__(self):
                    return str(
                        _to_internal_camera_state_network_local_interface_ipv4(self)
                    )

            def __init__(
                self,
                interface_name=_zivid.CameraState.Network.LocalInterface.InterfaceName().value,
                ipv4=None,
            ):

                if isinstance(interface_name, (str,)):
                    self._interface_name = (
                        _zivid.CameraState.Network.LocalInterface.InterfaceName(
                            interface_name
                        )
                    )
                else:
                    raise TypeError(
                        "Unsupported type, expected: (str,), got {value_type}".format(
                            value_type=type(interface_name)
                        )
                    )

                if ipv4 is None:
                    ipv4 = self.IPV4()
                if not isinstance(ipv4, self.IPV4):
                    raise TypeError(
                        "Unsupported type: {value}".format(value=type(ipv4))
                    )
                self._ipv4 = ipv4

            @property
            def interface_name(self):
                return self._interface_name.value

            @property
            def ipv4(self):
                return self._ipv4

            @interface_name.setter
            def interface_name(self, value):
                if isinstance(value, (str,)):
                    self._interface_name = (
                        _zivid.CameraState.Network.LocalInterface.InterfaceName(value)
                    )
                else:
                    raise TypeError(
                        "Unsupported type, expected: str, got {value_type}".format(
                            value_type=type(value)
                        )
                    )

            @ipv4.setter
            def ipv4(self, value):
                if not isinstance(value, self.IPV4):
                    raise TypeError(
                        "Unsupported type {value}".format(value=type(value))
                    )
                self._ipv4 = value

            def __eq__(self, other):
                if (
                    self._interface_name == other._interface_name
                    and self._ipv4 == other._ipv4
                ):
                    return True
                return False

            def __str__(self):
                return str(_to_internal_camera_state_network_local_interface(self))

        class IPV4:

            def __init__(
                self,
                address=_zivid.CameraState.Network.IPV4.Address().value,
            ):

                if isinstance(address, (str,)):
                    self._address = _zivid.CameraState.Network.IPV4.Address(address)
                else:
                    raise TypeError(
                        "Unsupported type, expected: (str,), got {value_type}".format(
                            value_type=type(address)
                        )
                    )

            @property
            def address(self):
                return self._address.value

            @address.setter
            def address(self, value):
                if isinstance(value, (str,)):
                    self._address = _zivid.CameraState.Network.IPV4.Address(value)
                else:
                    raise TypeError(
                        "Unsupported type, expected: str, got {value_type}".format(
                            value_type=type(value)
                        )
                    )

            def __eq__(self, other):
                if self._address == other._address:
                    return True
                return False

            def __str__(self):
                return str(_to_internal_camera_state_network_ipv4(self))

        def __init__(
            self,
            local_interfaces=None,
            ipv4=None,
        ):

            if local_interfaces is None:
                self._local_interfaces = []
            elif isinstance(local_interfaces, (collections.abc.Iterable,)):
                self._local_interfaces = []
                for item in local_interfaces:
                    if isinstance(item, self.LocalInterface):
                        self._local_interfaces.append(item)
                    else:
                        raise TypeError(
                            "Unsupported type {item_type}".format(item_type=type(item))
                        )
            else:
                raise TypeError(
                    "Unsupported type, expected: (collections.abc.Iterable,) or None, got {value_type}".format(
                        value_type=type(local_interfaces)
                    )
                )

            if ipv4 is None:
                ipv4 = self.IPV4()
            if not isinstance(ipv4, self.IPV4):
                raise TypeError("Unsupported type: {value}".format(value=type(ipv4)))
            self._ipv4 = ipv4

        @property
        def local_interfaces(self):
            return self._local_interfaces

        @property
        def ipv4(self):
            return self._ipv4

        @local_interfaces.setter
        def local_interfaces(self, value):
            if not isinstance(value, (collections.abc.Iterable,)):
                raise TypeError("Unsupported type {value}".format(value=type(value)))
            self._local_interfaces = []
            for item in value:
                if isinstance(item, self.LocalInterface):
                    self._local_interfaces.append(item)
                else:
                    raise TypeError(
                        "Unsupported type {item_type}".format(item_type=type(item))
                    )

        @ipv4.setter
        def ipv4(self, value):
            if not isinstance(value, self.IPV4):
                raise TypeError("Unsupported type {value}".format(value=type(value)))
            self._ipv4 = value

        def __eq__(self, other):
            if (
                self._local_interfaces == other._local_interfaces
                and self._ipv4 == other._ipv4
            ):
                return True
            return False

        def __str__(self):
            return str(_to_internal_camera_state_network(self))

    class Temperature:

        def __init__(
            self,
            dmd=_zivid.CameraState.Temperature.DMD().value,
            general=_zivid.CameraState.Temperature.General().value,
            led=_zivid.CameraState.Temperature.LED().value,
            lens=_zivid.CameraState.Temperature.Lens().value,
            pcb=_zivid.CameraState.Temperature.PCB().value,
        ):

            if isinstance(
                dmd,
                (
                    float,
                    int,
                ),
            ):
                self._dmd = _zivid.CameraState.Temperature.DMD(dmd)
            else:
                raise TypeError(
                    "Unsupported type, expected: (float, int,), got {value_type}".format(
                        value_type=type(dmd)
                    )
                )

            if isinstance(
                general,
                (
                    float,
                    int,
                ),
            ):
                self._general = _zivid.CameraState.Temperature.General(general)
            else:
                raise TypeError(
                    "Unsupported type, expected: (float, int,), got {value_type}".format(
                        value_type=type(general)
                    )
                )

            if isinstance(
                led,
                (
                    float,
                    int,
                ),
            ):
                self._led = _zivid.CameraState.Temperature.LED(led)
            else:
                raise TypeError(
                    "Unsupported type, expected: (float, int,), got {value_type}".format(
                        value_type=type(led)
                    )
                )

            if isinstance(
                lens,
                (
                    float,
                    int,
                ),
            ):
                self._lens = _zivid.CameraState.Temperature.Lens(lens)
            else:
                raise TypeError(
                    "Unsupported type, expected: (float, int,), got {value_type}".format(
                        value_type=type(lens)
                    )
                )

            if isinstance(
                pcb,
                (
                    float,
                    int,
                ),
            ):
                self._pcb = _zivid.CameraState.Temperature.PCB(pcb)
            else:
                raise TypeError(
                    "Unsupported type, expected: (float, int,), got {value_type}".format(
                        value_type=type(pcb)
                    )
                )

        @property
        def dmd(self):
            return self._dmd.value

        @property
        def general(self):
            return self._general.value

        @property
        def led(self):
            return self._led.value

        @property
        def lens(self):
            return self._lens.value

        @property
        def pcb(self):
            return self._pcb.value

        @dmd.setter
        def dmd(self, value):
            if isinstance(
                value,
                (
                    float,
                    int,
                ),
            ):
                self._dmd = _zivid.CameraState.Temperature.DMD(value)
            else:
                raise TypeError(
                    "Unsupported type, expected: float or  int, got {value_type}".format(
                        value_type=type(value)
                    )
                )

        @general.setter
        def general(self, value):
            if isinstance(
                value,
                (
                    float,
                    int,
                ),
            ):
                self._general = _zivid.CameraState.Temperature.General(value)
            else:
                raise TypeError(
                    "Unsupported type, expected: float or  int, got {value_type}".format(
                        value_type=type(value)
                    )
                )

        @led.setter
        def led(self, value):
            if isinstance(
                value,
                (
                    float,
                    int,
                ),
            ):
                self._led = _zivid.CameraState.Temperature.LED(value)
            else:
                raise TypeError(
                    "Unsupported type, expected: float or  int, got {value_type}".format(
                        value_type=type(value)
                    )
                )

        @lens.setter
        def lens(self, value):
            if isinstance(
                value,
                (
                    float,
                    int,
                ),
            ):
                self._lens = _zivid.CameraState.Temperature.Lens(value)
            else:
                raise TypeError(
                    "Unsupported type, expected: float or  int, got {value_type}".format(
                        value_type=type(value)
                    )
                )

        @pcb.setter
        def pcb(self, value):
            if isinstance(
                value,
                (
                    float,
                    int,
                ),
            ):
                self._pcb = _zivid.CameraState.Temperature.PCB(value)
            else:
                raise TypeError(
                    "Unsupported type, expected: float or  int, got {value_type}".format(
                        value_type=type(value)
                    )
                )

        def __eq__(self, other):
            if (
                self._dmd == other._dmd
                and self._general == other._general
                and self._led == other._led
                and self._lens == other._lens
                and self._pcb == other._pcb
            ):
                return True
            return False

        def __str__(self):
            return str(_to_internal_camera_state_temperature(self))

    class InaccessibleReason:

        ipConflictWithAnotherCamera = "ipConflictWithAnotherCamera"
        ipConflictWithLocalNetworkAdapter = "ipConflictWithLocalNetworkAdapter"
        ipInMultipleLocalSubnets = "ipInMultipleLocalSubnets"
        ipNotInLocalSubnet = "ipNotInLocalSubnet"

        _valid_values = {
            "ipConflictWithAnotherCamera": _zivid.CameraState.InaccessibleReason.ipConflictWithAnotherCamera,
            "ipConflictWithLocalNetworkAdapter": _zivid.CameraState.InaccessibleReason.ipConflictWithLocalNetworkAdapter,
            "ipInMultipleLocalSubnets": _zivid.CameraState.InaccessibleReason.ipInMultipleLocalSubnets,
            "ipNotInLocalSubnet": _zivid.CameraState.InaccessibleReason.ipNotInLocalSubnet,
        }

        @classmethod
        def valid_values(cls):
            return list(cls._valid_values.keys())

    class Status:

        applyingNetworkConfiguration = "applyingNetworkConfiguration"
        available = "available"
        busy = "busy"
        connected = "connected"
        connecting = "connecting"
        disappeared = "disappeared"
        disconnecting = "disconnecting"
        firmwareUpdateRequired = "firmwareUpdateRequired"
        inaccessible = "inaccessible"
        updatingFirmware = "updatingFirmware"

        _valid_values = {
            "applyingNetworkConfiguration": _zivid.CameraState.Status.applyingNetworkConfiguration,
            "available": _zivid.CameraState.Status.available,
            "busy": _zivid.CameraState.Status.busy,
            "connected": _zivid.CameraState.Status.connected,
            "connecting": _zivid.CameraState.Status.connecting,
            "disappeared": _zivid.CameraState.Status.disappeared,
            "disconnecting": _zivid.CameraState.Status.disconnecting,
            "firmwareUpdateRequired": _zivid.CameraState.Status.firmwareUpdateRequired,
            "inaccessible": _zivid.CameraState.Status.inaccessible,
            "updatingFirmware": _zivid.CameraState.Status.updatingFirmware,
        }

        @classmethod
        def valid_values(cls):
            return list(cls._valid_values.keys())

    def __init__(
        self,
        available=_zivid.CameraState.Available().value,
        connected=_zivid.CameraState.Connected().value,
        inaccessible_reason=_zivid.CameraState.InaccessibleReason().value,
        status=_zivid.CameraState.Status().value,
        network=None,
        temperature=None,
    ):

        if isinstance(available, (bool,)):
            self._available = _zivid.CameraState.Available(available)
        else:
            raise TypeError(
                "Unsupported type, expected: (bool,), got {value_type}".format(
                    value_type=type(available)
                )
            )

        if isinstance(connected, (bool,)):
            self._connected = _zivid.CameraState.Connected(connected)
        else:
            raise TypeError(
                "Unsupported type, expected: (bool,), got {value_type}".format(
                    value_type=type(connected)
                )
            )

        if (
            isinstance(inaccessible_reason, _zivid.CameraState.InaccessibleReason.enum)
            or inaccessible_reason is None
        ):
            self._inaccessible_reason = _zivid.CameraState.InaccessibleReason(
                inaccessible_reason
            )
        elif isinstance(inaccessible_reason, str):
            self._inaccessible_reason = _zivid.CameraState.InaccessibleReason(
                self.InaccessibleReason._valid_values[inaccessible_reason]
            )
        else:
            raise TypeError(
                "Unsupported type, expected: str or None, got {value_type}".format(
                    value_type=type(inaccessible_reason)
                )
            )

        if isinstance(status, _zivid.CameraState.Status.enum):
            self._status = _zivid.CameraState.Status(status)
        elif isinstance(status, str):
            self._status = _zivid.CameraState.Status(self.Status._valid_values[status])
        else:
            raise TypeError(
                "Unsupported type, expected: str, got {value_type}".format(
                    value_type=type(status)
                )
            )

        if network is None:
            network = self.Network()
        if not isinstance(network, self.Network):
            raise TypeError("Unsupported type: {value}".format(value=type(network)))
        self._network = network

        if temperature is None:
            temperature = self.Temperature()
        if not isinstance(temperature, self.Temperature):
            raise TypeError("Unsupported type: {value}".format(value=type(temperature)))
        self._temperature = temperature

    @property
    def available(self):
        return self._available.value

    @property
    def connected(self):
        return self._connected.value

    @property
    def inaccessible_reason(self):
        if self._inaccessible_reason.value is None:
            return None
        for key, internal_value in self.InaccessibleReason._valid_values.items():
            if internal_value == self._inaccessible_reason.value:
                return key
        raise ValueError(
            "Unsupported value {value}".format(value=self._inaccessible_reason)
        )

    @property
    def status(self):
        if self._status.value is None:
            return None
        for key, internal_value in self.Status._valid_values.items():
            if internal_value == self._status.value:
                return key
        raise ValueError("Unsupported value {value}".format(value=self._status))

    @property
    def network(self):
        return self._network

    @property
    def temperature(self):
        return self._temperature

    @available.setter
    def available(self, value):
        if isinstance(value, (bool,)):
            self._available = _zivid.CameraState.Available(value)
        else:
            raise TypeError(
                "Unsupported type, expected: bool, got {value_type}".format(
                    value_type=type(value)
                )
            )

    @connected.setter
    def connected(self, value):
        if isinstance(value, (bool,)):
            self._connected = _zivid.CameraState.Connected(value)
        else:
            raise TypeError(
                "Unsupported type, expected: bool, got {value_type}".format(
                    value_type=type(value)
                )
            )

    @inaccessible_reason.setter
    def inaccessible_reason(self, value):
        if isinstance(value, str):
            self._inaccessible_reason = _zivid.CameraState.InaccessibleReason(
                self.InaccessibleReason._valid_values[value]
            )
        elif (
            isinstance(value, _zivid.CameraState.InaccessibleReason.enum)
            or value is None
        ):
            self._inaccessible_reason = _zivid.CameraState.InaccessibleReason(value)
        else:
            raise TypeError(
                "Unsupported type, expected: str or None, got {value_type}".format(
                    value_type=type(value)
                )
            )

    @status.setter
    def status(self, value):
        if isinstance(value, str):
            self._status = _zivid.CameraState.Status(self.Status._valid_values[value])
        elif isinstance(value, _zivid.CameraState.Status.enum):
            self._status = _zivid.CameraState.Status(value)
        else:
            raise TypeError(
                "Unsupported type, expected: str, got {value_type}".format(
                    value_type=type(value)
                )
            )

    @network.setter
    def network(self, value):
        if not isinstance(value, self.Network):
            raise TypeError("Unsupported type {value}".format(value=type(value)))
        self._network = value

    @temperature.setter
    def temperature(self, value):
        if not isinstance(value, self.Temperature):
            raise TypeError("Unsupported type {value}".format(value=type(value)))
        self._temperature = value

    @classmethod
    def load(cls, file_name):
        return _to_camera_state(_zivid.CameraState(str(file_name)))

    def save(self, file_name):
        _to_internal_camera_state(self).save(str(file_name))

    @classmethod
    def from_serialized(cls, value):
        return _to_camera_state(_zivid.CameraState.from_serialized(str(value)))

    def serialize(self):
        return _to_internal_camera_state(self).serialize()

    def __eq__(self, other):
        if (
            self._available == other._available
            and self._connected == other._connected
            and self._inaccessible_reason == other._inaccessible_reason
            and self._status == other._status
            and self._network == other._network
            and self._temperature == other._temperature
        ):
            return True
        return False

    def __str__(self):
        return str(_to_internal_camera_state(self))


def _to_camera_state_network_local_interface_ipv4_subnet(internal_subnet):
    return CameraState.Network.LocalInterface.IPV4.Subnet(
        address=internal_subnet.address.value,
        mask=internal_subnet.mask.value,
    )


def _to_camera_state_network_local_interface_ipv4(internal_ipv4):
    return CameraState.Network.LocalInterface.IPV4(
        subnets=[
            _to_camera_state_network_local_interface_ipv4_subnet(value)
            for value in internal_ipv4.subnets.value
        ],
    )


def _to_camera_state_network_local_interface(internal_local_interface):
    return CameraState.Network.LocalInterface(
        ipv4=_to_camera_state_network_local_interface_ipv4(
            internal_local_interface.ipv4
        ),
        interface_name=internal_local_interface.interface_name.value,
    )


def _to_camera_state_network_ipv4(internal_ipv4):
    return CameraState.Network.IPV4(
        address=internal_ipv4.address.value,
    )


def _to_camera_state_network(internal_network):
    return CameraState.Network(
        local_interfaces=[
            _to_camera_state_network_local_interface(value)
            for value in internal_network.local_interfaces.value
        ],
        ipv4=_to_camera_state_network_ipv4(internal_network.ipv4),
    )


def _to_camera_state_temperature(internal_temperature):
    return CameraState.Temperature(
        dmd=internal_temperature.dmd.value,
        general=internal_temperature.general.value,
        led=internal_temperature.led.value,
        lens=internal_temperature.lens.value,
        pcb=internal_temperature.pcb.value,
    )


def _to_camera_state(internal_camera_state):
    return CameraState(
        network=_to_camera_state_network(internal_camera_state.network),
        temperature=_to_camera_state_temperature(internal_camera_state.temperature),
        available=internal_camera_state.available.value,
        connected=internal_camera_state.connected.value,
        inaccessible_reason=internal_camera_state.inaccessible_reason.value,
        status=internal_camera_state.status.value,
    )


def _to_internal_camera_state_network_local_interface_ipv4_subnet(subnet):
    internal_subnet = _zivid.CameraState.Network.LocalInterface.IPV4.Subnet()

    internal_subnet.address = (
        _zivid.CameraState.Network.LocalInterface.IPV4.Subnet.Address(subnet.address)
    )
    internal_subnet.mask = _zivid.CameraState.Network.LocalInterface.IPV4.Subnet.Mask(
        subnet.mask
    )

    return internal_subnet


def _to_internal_camera_state_network_local_interface_ipv4(ipv4):
    internal_ipv4 = _zivid.CameraState.Network.LocalInterface.IPV4()

    temp_subnets = _zivid.CameraState.Network.LocalInterface.IPV4.Subnets()
    for value in ipv4.subnets:
        temp_subnets.append(
            _to_internal_camera_state_network_local_interface_ipv4_subnet(value)
        )
    internal_ipv4.subnets = temp_subnets

    return internal_ipv4


def _to_internal_camera_state_network_local_interface(local_interface):
    internal_local_interface = _zivid.CameraState.Network.LocalInterface()

    internal_local_interface.interface_name = (
        _zivid.CameraState.Network.LocalInterface.InterfaceName(
            local_interface.interface_name
        )
    )

    internal_local_interface.ipv4 = (
        _to_internal_camera_state_network_local_interface_ipv4(local_interface.ipv4)
    )
    return internal_local_interface


def _to_internal_camera_state_network_ipv4(ipv4):
    internal_ipv4 = _zivid.CameraState.Network.IPV4()

    internal_ipv4.address = _zivid.CameraState.Network.IPV4.Address(ipv4.address)

    return internal_ipv4


def _to_internal_camera_state_network(network):
    internal_network = _zivid.CameraState.Network()

    temp_local_interfaces = _zivid.CameraState.Network.LocalInterfaces()
    for value in network.local_interfaces:
        temp_local_interfaces.append(
            _to_internal_camera_state_network_local_interface(value)
        )
    internal_network.local_interfaces = temp_local_interfaces

    internal_network.ipv4 = _to_internal_camera_state_network_ipv4(network.ipv4)
    return internal_network


def _to_internal_camera_state_temperature(temperature):
    internal_temperature = _zivid.CameraState.Temperature()

    internal_temperature.dmd = _zivid.CameraState.Temperature.DMD(temperature.dmd)
    internal_temperature.general = _zivid.CameraState.Temperature.General(
        temperature.general
    )
    internal_temperature.led = _zivid.CameraState.Temperature.LED(temperature.led)
    internal_temperature.lens = _zivid.CameraState.Temperature.Lens(temperature.lens)
    internal_temperature.pcb = _zivid.CameraState.Temperature.PCB(temperature.pcb)

    return internal_temperature


def _to_internal_camera_state(camera_state):
    internal_camera_state = _zivid.CameraState()

    internal_camera_state.available = _zivid.CameraState.Available(
        camera_state.available
    )
    internal_camera_state.connected = _zivid.CameraState.Connected(
        camera_state.connected
    )
    internal_camera_state.inaccessible_reason = _zivid.CameraState.InaccessibleReason(
        camera_state._inaccessible_reason.value
    )
    internal_camera_state.status = _zivid.CameraState.Status(camera_state._status.value)

    internal_camera_state.network = _to_internal_camera_state_network(
        camera_state.network
    )
    internal_camera_state.temperature = _to_internal_camera_state_temperature(
        camera_state.temperature
    )
    return internal_camera_state



"""Module for experimental pixel mapping. This API may change in the future."""

import _zivid


class PixelMapping:
    """Pixel mapping from subsampled to full resolution.

    Required when mapping an index in a subsampled point cloud to e.g. a full resolution 2D image.
    """

    def __init__(self, row_stride=1, col_stride=1, row_offset=0.0, col_offset=0.0):
        self.__impl = _zivid.PixelMapping(
            row_stride, col_stride, row_offset, col_offset
        )

    @property
    def row_stride(self):
        return self.__impl.row_stride()

    @property
    def col_stride(self):
        return self.__impl.col_stride()

    @property
    def row_offset(self):
        return self.__impl.row_offset()

    @property
    def col_offset(self):
        return self.__impl.col_offset()

    def __str__(self):
        return str(self.__impl)



"""Module defining file formats that point cloud data can be exported to."""

import _zivid


class ColorSpace:  # pylint: disable=too-few-public-methods
    """Color space for saving point cloud."""

    linear_rgb = "linear_rgb"
    srgb = "srgb"

    @staticmethod
    def valid_values():
        """Get valid values for color space.

        Returns:
            List of valid color spaces.
        """
        return [ColorSpace.linear_rgb, ColorSpace.srgb]

    @classmethod
    def _to_internal(cls, value):
        if value == ColorSpace.linear_rgb:
            return _zivid.point_cloud_export.ColorSpace.linear_rgb
        if value == ColorSpace.srgb:
            return _zivid.point_cloud_export.ColorSpace.srgb
        raise ValueError(
            "Invalid color space '{}'. Valid color spaces are: {}".format(
                value, cls.valid_values()
            )
        )


class ZDF:  # pylint: disable=too-few-public-methods
    """Specification for saving frame in ZDF (*.zdf) format."""

    def __init__(self, file_name):
        """Create a ZDF file format specification with file name.

        Args:
            file_name: File name.

        Raises:
            TypeError: If file_name is not a string.
        """
        if not isinstance(file_name, str):
            raise TypeError(
                "Unsupported type for argument file_name. Got {}, expected {}".format(
                    type(file_name), str
                )
            )
        self.__impl = _zivid.point_cloud_export.file_format.ZDF(file_name)

    def __str__(self):
        return str(self.__impl)


class PLY:  # pylint: disable=too-few-public-methods
    """Specification for saving frame in PLY (*.ply) format.

    PLY is a file format developed at Stanford. To learn more about the PLY file format,
    see https://paulbourke.net/dataformats/ply/.
    """

    class Layout:
        """Layout for saving point cloud."""

        ordered = "ordered"
        unordered = "unordered"

        @staticmethod
        def valid_values():
            """Get valid values for layout.

            Returns:
                List of valid layouts.
            """
            return [PLY.Layout.ordered, PLY.Layout.unordered]

        @classmethod
        def _to_internal(cls, value):
            if value == PLY.Layout.ordered:
                return _zivid.point_cloud_export.file_format.PLY.Layout.ordered
            if value == PLY.Layout.unordered:
                return _zivid.point_cloud_export.file_format.PLY.Layout.unordered
            raise ValueError(
                "Invalid layout '{}'. Valid layouts are: {}".format(
                    value, cls.valid_values()
                )
            )

    def __init__(self, file_name, layout=Layout.ordered, color_space=ColorSpace.srgb):
        """Create a PLY file format specification with file name.

        Args:
            file_name: File name.
            layout: Layout of point cloud. Default is ordered.
            color_space: Color space of point cloud. Default is sRGB.

        Raises:
            TypeError: If file_name, layout, or color_space are not strings.
        """
        if not isinstance(file_name, str):
            raise TypeError(
                "Unsupported type for argument file_name. Got {}, expected {}".format(
                    type(file_name), str
                )
            )
        if not isinstance(layout, str):
            raise TypeError(
                "Unsupported type for argument layout. Got {}, expected {}".format(
                    type(layout), str
                )
            )
        if not isinstance(color_space, str):
            raise TypeError(
                "Unsupported type for argument color_space. Got {}, expected {}".format(
                    type(color_space), str
                )
            )
        self.__impl = _zivid.point_cloud_export.file_format.PLY(
            file_name,
            PLY.Layout._to_internal(layout),
            ColorSpace._to_internal(color_space),
        )

    def __str__(self):
        return str(self.__impl)


class XYZ:  # pylint: disable=too-few-public-methods
    """Specification for saving frame in ASCII (*.xyz) format.

    ASCII characters are used to store cartesian coordinates of XYZ points and RGB color values.
    """

    def __init__(self, file_name, color_space=ColorSpace.srgb):
        """Create a XYZ file format specification with file name.

        Sets color space to linear RGB.

        Args:
            file_name: File name.
            color_space: Color space of point cloud. Default is sRGB.

        Raises:
            TypeError: If file_name or color_space are not strings.
        """
        if not isinstance(file_name, str):
            raise TypeError(
                "Unsupported type for argument file_name. Got {}, expected {}".format(
                    type(file_name), str
                )
            )
        if not isinstance(color_space, str):
            raise TypeError(
                "Unsupported type for argument color_space. Got {}, expected {}".format(
                    type(color_space), str
                )
            )
        self.__impl = _zivid.point_cloud_export.file_format.XYZ(
            file_name, ColorSpace._to_internal(color_space)
        )

    def __str__(self):
        return str(self.__impl)


class PCD:  # pylint: disable=too-few-public-methods
    """Specification for saving frame in PCD (*.pcd) format.

    PCD is a file format native to the Point Cloud Library (PCL). To learn more about
    the PCD file format, see
    https://pcl.readthedocs.io/projects/tutorials/en/latest/pcd_file_format.html#pcd-file-format.
    """

    def __init__(self, file_name, color_space=ColorSpace.srgb):
        """Create a PCD file format specification with file name.

        Args:
            file_name: File name.
            color_space: Color space of point cloud. Default is sRGB.

        Raises:
            TypeError: If file_name or color_space are not strings.
        """
        if not isinstance(file_name, str):
            raise TypeError(
                "Unsupported type for argument file_name. Got {}, expected {}".format(
                    type(file_name), str
                )
            )
        if not isinstance(color_space, str):
            raise TypeError(
                "Unsupported type for argument color_space. Got {}, expected {}".format(
                    type(color_space), str
                )
            )
        self.__impl = _zivid.point_cloud_export.file_format.PCD(
            file_name,
            ColorSpace._to_internal(color_space),
        )

    def __str__(self):
        return str(self.__impl)
